@@ -1,6 +1,6 @@
-/* Copyright (c) 2015-2025 The Khronos Group Inc.
- * Copyright (c) 2015-2025 Valve Corporation
- * Copyright (c) 2015-2025 LunarG, Inc.
+/* Copyright (c) 2015-2026 The Khronos Group Inc.
+ * Copyright (c) 2015-2026 Valve Corporation
+ * Copyright (c) 2015-2026 LunarG, Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
@@ -19,9 +19,11 @@
 
 #include "best_practices/best_practices_validation.h"
 #include "best_practices/bp_state.h"
+#include "generated/spirv_grammar_helper.h"
 #include "state_tracker/render_pass_state.h"
 #include "state_tracker/pipeline_state.h"
 #include "chassis/chassis_modification_state.h"
+#include "utils/math_utils.h"
 
 static inline bool FormatHasFullThroughputBlendingArm(VkFormat format) {
     switch (format) {
@@ -41,33 +43,36 @@ static inline bool FormatHasFullThroughputBlendingArm(VkFormat format) {
     }
 }
 
-bool BestPractices::ValidateMultisampledBlendingArm(const VkGraphicsPipelineCreateInfo& create_info,
-                                                    const Location& create_info_loc) const {
+bool BestPractices::ValidateMultisampledBlendingArm(const vvl::Pipeline& pipeline, const Location& create_info_loc) const {
     bool skip = false;
 
-    if (!create_info.pColorBlendState || !create_info.pMultisampleState ||
-        create_info.pMultisampleState->rasterizationSamples == VK_SAMPLE_COUNT_1_BIT ||
-        create_info.pMultisampleState->sampleShadingEnable) {
+    const auto* color_blend_state = pipeline.ColorBlendState();
+    const auto* ms_state = pipeline.MultisampleState();
+    if (!color_blend_state || !ms_state || ms_state->rasterizationSamples == VK_SAMPLE_COUNT_1_BIT ||
+        ms_state->sampleShadingEnable) {
         return skip;
     }
 
-    auto rp_state = Get<vvl::RenderPass>(create_info.renderPass);
-    if (!rp_state) return skip;
+    auto rp_state = Get<vvl::RenderPass>(pipeline.GraphicsCreateInfo().renderPass);
+    if (!rp_state) {
+        return skip;
+    }
 
-    const auto& subpass = rp_state->create_info.pSubpasses[create_info.subpass];
+    const auto& subpass = rp_state->create_info.pSubpasses[pipeline.Subpass()];
 
     // According to spec, pColorBlendState must be ignored if subpass does not have color attachments.
-    uint32_t num_color_attachments = std::min(subpass.colorAttachmentCount, create_info.pColorBlendState->attachmentCount);
+    uint32_t num_color_attachments = std::min(subpass.colorAttachmentCount, color_blend_state->attachmentCount);
 
     for (uint32_t j = 0; j < num_color_attachments; j++) {
-        const auto& blend_att = create_info.pColorBlendState->pAttachments[j];
+        const auto& blend_att = color_blend_state->pAttachments[j];
         uint32_t att = subpass.pColorAttachments[j].attachment;
 
         if (att != VK_ATTACHMENT_UNUSED && blend_att.blendEnable && blend_att.colorWriteMask) {
             if (!FormatHasFullThroughputBlendingArm(rp_state->create_info.pAttachments[att].format)) {
                 skip |= LogPerformanceWarning("BestPractices-Arm-vkCreatePipelines-multisampled-blending", device, create_info_loc,
                                               "%s Pipeline is multisampled and "
-                                              "color attachment #%u makes use "
+                                              "color attachment %" PRIu32
+                                              " makes use "
                                               "of a format which cannot be blended at full throughput when using MSAA.",
                                               VendorSpecificTag(kBPVendorArm), j);
             }
@@ -94,26 +99,29 @@ void BestPractices::ManualPostCallRecordCreateComputePipelines(VkDevice device,
 bool BestPractices::ValidateCreateGraphicsPipeline(const VkGraphicsPipelineCreateInfo& create_info, const vvl::Pipeline& pipeline,
                                                    const Location create_info_loc) const {
     bool skip = false;
-    if (!(pipeline.active_shaders & VK_SHADER_STAGE_MESH_BIT_EXT) && create_info.pVertexInputState) {
-        const auto& vertex_input = *create_info.pVertexInputState;
+
+    const auto* vertex_input = pipeline.InputState();
+    if (!(pipeline.active_shaders & VK_SHADER_STAGE_MESH_BIT_EXT) && vertex_input) {
         uint32_t count = 0;
-        for (uint32_t j = 0; j < vertex_input.vertexBindingDescriptionCount; j++) {
-            if (vertex_input.pVertexBindingDescriptions[j].inputRate == VK_VERTEX_INPUT_RATE_INSTANCE) {
+        for (uint32_t j = 0; j < vertex_input->vertexBindingDescriptionCount; j++) {
+            if (vertex_input->pVertexBindingDescriptions[j].inputRate == VK_VERTEX_INPUT_RATE_INSTANCE) {
                 count++;
             }
         }
         if (count > kMaxInstancedVertexBuffers) {
-            skip |= LogPerformanceWarning(
-                "BestPractices-vkCreateGraphicsPipelines-too-many-instanced-vertex-buffers", device, create_info_loc,
-                "The pipeline is using %u instanced vertex buffers (current limit: %u), but this can be inefficient on the "
-                "GPU. If using instanced vertex attributes prefer interleaving them in a single buffer.",
-                count, kMaxInstancedVertexBuffers);
+            skip |= LogPerformanceWarning("BestPractices-vkCreateGraphicsPipelines-too-many-instanced-vertex-buffers", device,
+                                          create_info_loc,
+                                          "The pipeline is using %" PRIu32 " instanced vertex buffers (current limit: %" PRIu32
+                                          "), but this can be inefficient on the "
+                                          "GPU. If using instanced vertex attributes prefer interleaving them in a single buffer.",
+                                          count, kMaxInstancedVertexBuffers);
         }
     }
 
-    if ((create_info.pRasterizationState) && (create_info.pRasterizationState->depthBiasEnable) &&
-        (create_info.pRasterizationState->depthBiasConstantFactor == 0.0f) &&
-        (create_info.pRasterizationState->depthBiasSlopeFactor == 0.0f) && VendorCheckEnabled(kBPVendorArm)) {
+    const auto* raster_state = pipeline.RasterizationState();
+
+    if (raster_state && raster_state->depthBiasEnable && raster_state->depthBiasConstantFactor == 0.0f &&
+        raster_state->depthBiasSlopeFactor == 0.0f && VendorCheckEnabled(kBPVendorArm)) {
         skip |=
             LogPerformanceWarning("BestPractices-Arm-vkCreatePipelines-depthbias-zero", device, create_info_loc,
                                   "%s This vkCreateGraphicsPipelines call is created with depthBiasEnable set to true "
@@ -135,12 +143,14 @@ bool BestPractices::ValidateCreateGraphicsPipeline(const VkGraphicsPipelineCreat
     }
 
     if (VendorCheckEnabled(kBPVendorArm)) {
-        skip |= ValidateMultisampledBlendingArm(create_info, create_info_loc);
+        skip |= ValidateMultisampledBlendingArm(pipeline, create_info_loc);
     }
 
     if (VendorCheckEnabled(kBPVendorAMD)) {
-        if (create_info.pInputAssemblyState && create_info.pInputAssemblyState->primitiveRestartEnable) {
-            skip |= LogPerformanceWarning("BestPractices-AMD-CreatePipelines-AvoidPrimitiveRestart", device, create_info_loc,
+        const auto* ia_state = pipeline.InputAssemblyState();
+        if (ia_state && ia_state->primitiveRestartEnable) {
+            skip |= LogPerformanceWarning("BestPractices-AMD-CreatePipelines-AvoidPrimitiveRestart", device,
+                                          create_info_loc.dot(Field::pInputAssemblyState).dot(Field::primitiveRestartEnable),
                                           "%s Use of primitive restart is not recommended", VendorSpecificTag(kBPVendorAMD));
         }
 
@@ -152,6 +162,15 @@ bool BestPractices::ValidateCreateGraphicsPipeline(const VkGraphicsPipelineCreat
         }
     }
 
+    for (uint32_t i = 0; i < pipeline.stage_states.size(); i++) {
+        auto& stage_state = pipeline.stage_states[i];
+        const VkShaderStageFlagBits stage = stage_state.GetStage();
+        // Only validate the shader state once when added, not again when linked
+        if ((stage & pipeline.linking_shaders) == 0) {
+            skip |= ValidateShaderStage(stage_state, &pipeline, create_info_loc.dot(Field::pStages, i));
+        }
+    }
+
     return skip;
 }
 
@@ -199,48 +218,6 @@ bool BestPractices::PreCallValidateCreateGraphicsPipelines(VkDevice device, VkPi
     return skip;
 }
 
-static std::vector<bp_state::AttachmentInfo> GetAttachmentAccess(vvl::Pipeline& pipe_state) {
-    std::vector<bp_state::AttachmentInfo> result;
-    auto rp = pipe_state.RenderPassState();
-    if (!rp || rp->UsesDynamicRendering()) {
-        return result;
-    }
-    const auto& create_info = pipe_state.GraphicsCreateInfo();
-    const auto& subpass = rp->create_info.pSubpasses[create_info.subpass];
-
-    // NOTE: see PIPELINE_LAYOUT and vku::safe_VkGraphicsPipelineCreateInfo constructors. pColorBlendState and pDepthStencilState
-    // are only non-null if they are enabled.
-    if (create_info.pColorBlendState && !(pipe_state.ignore_color_attachments)) {
-        // According to spec, pColorBlendState must be ignored if subpass does not have color attachments.
-        uint32_t num_color_attachments = std::min(subpass.colorAttachmentCount, create_info.pColorBlendState->attachmentCount);
-        for (uint32_t j = 0; j < num_color_attachments; j++) {
-            if (create_info.pColorBlendState->pAttachments[j].colorWriteMask != 0) {
-                uint32_t attachment = subpass.pColorAttachments[j].attachment;
-                if (attachment != VK_ATTACHMENT_UNUSED) {
-                    result.emplace_back(attachment, VK_IMAGE_ASPECT_COLOR_BIT);
-                }
-            }
-        }
-    }
-
-    if (create_info.pDepthStencilState &&
-        (create_info.pDepthStencilState->depthTestEnable || create_info.pDepthStencilState->depthBoundsTestEnable ||
-         create_info.pDepthStencilState->stencilTestEnable)) {
-        uint32_t attachment = subpass.pDepthStencilAttachment ? subpass.pDepthStencilAttachment->attachment : VK_ATTACHMENT_UNUSED;
-        if (attachment != VK_ATTACHMENT_UNUSED) {
-            VkImageAspectFlags aspects = 0;
-            if (create_info.pDepthStencilState->depthTestEnable || create_info.pDepthStencilState->depthBoundsTestEnable) {
-                aspects |= VK_IMAGE_ASPECT_DEPTH_BIT;
-            }
-            if (create_info.pDepthStencilState->stencilTestEnable) {
-                aspects |= VK_IMAGE_ASPECT_STENCIL_BIT;
-            }
-            result.emplace_back(attachment, aspects);
-        }
-    }
-    return result;
-}
-
 void BestPractices::ManualPostCallRecordCreateGraphicsPipelines(VkDevice device, VkPipelineCache pipelineCache, uint32_t count,
                                                                 const VkGraphicsPipelineCreateInfo* pCreateInfos,
                                                                 const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
@@ -264,8 +241,7 @@ bool BestPractices::PreCallValidateCreateComputePipelines(VkDevice device, VkPip
         skip |=
             LogPerformanceWarning("BestPractices-vkCreateComputePipelines-multiple-pipelines-no-cache", device, error_obj.location,
                                   "creating multiple pipelines (createInfoCount is %" PRIu32
-                                  ") but is not using a "
-                                  "pipeline cache, which may help with performance",
+                                  ") but is not using a pipeline cache, which may help with performance",
                                   createInfoCount);
     }
 
@@ -296,45 +272,38 @@ bool BestPractices::ValidateComputeShaderArm(const spirv::Module& module_state,
                                              const Location& loc) const {
     bool skip = false;
 
-    uint32_t x = {}, y = {}, z = {};
-    if (!module_state.FindLocalSize(entrypoint, x, y, z)) {
+    spirv::LocalSize local_size = module_state.FindLocalSize(entrypoint);
+    if (local_size.x == 0) {
         return false;
     }
 
-    const uint64_t thread_count = x * y * z;
+    const uint64_t thread_count = local_size.x * local_size.y * local_size.z;
 
     // Generate a priori warnings about work group sizes.
     if (thread_count > kMaxEfficientWorkGroupThreadCountArm) {
         skip |= LogPerformanceWarning(
             "BestPractices-Arm-vkCreateComputePipelines-compute-work-group-size", device, loc,
-            "%s compute shader with work group dimensions (%" PRIu32 ", %" PRIu32
-            ", "
-            "%" PRIu32 ") (%" PRIu64
+            "%s compute shader with work group dimensions (%s) (%" PRIu64
             " threads total), has more threads than advised in a single work group. It is advised to use work "
             "groups with less than %" PRIu32 " threads, especially when using barrier() or shared memory.",
-            VendorSpecificTag(kBPVendorArm), x, y, z, thread_count, kMaxEfficientWorkGroupThreadCountArm);
+            VendorSpecificTag(kBPVendorArm), local_size.ToString().c_str(), thread_count, kMaxEfficientWorkGroupThreadCountArm);
     }
 
-    if (thread_count == 1 || ((x > 1) && (x & (kThreadGroupDispatchCountAlignmentArm - 1))) ||
-        ((y > 1) && (y & (kThreadGroupDispatchCountAlignmentArm - 1))) ||
-        ((z > 1) && (z & (kThreadGroupDispatchCountAlignmentArm - 1)))) {
+    if (thread_count == 1 || ((local_size.x > 1) && (local_size.x & (kThreadGroupDispatchCountAlignmentArm - 1))) ||
+        ((local_size.y > 1) && (local_size.y & (kThreadGroupDispatchCountAlignmentArm - 1))) ||
+        ((local_size.z > 1) && (local_size.z & (kThreadGroupDispatchCountAlignmentArm - 1)))) {
         skip |= LogPerformanceWarning("BestPractices-Arm-vkCreateComputePipelines-compute-thread-group-alignment", device, loc,
-                                      "%s compute shader with work group dimensions (%" PRIu32
-                                      ", "
-                                      "%" PRIu32 ", %" PRIu32 ") is not aligned to %" PRIu32
-                                      " "
-                                      "threads. On Arm Mali architectures, not aligning work group sizes to %" PRIu32
-                                      " may "
-                                      "leave threads idle on the shader "
-                                      "core.",
-                                      VendorSpecificTag(kBPVendorArm), x, y, z, kThreadGroupDispatchCountAlignmentArm,
-                                      kThreadGroupDispatchCountAlignmentArm);
+                                      "%s compute shader with work group dimensions (%s) is not aligned to %" PRIu32
+                                      " threads. On Arm Mali architectures, not aligning work group sizes to %" PRIu32
+                                      " may leave threads idle on the shader core.",
+                                      VendorSpecificTag(kBPVendorArm), local_size.ToString().c_str(),
+                                      kThreadGroupDispatchCountAlignmentArm, kThreadGroupDispatchCountAlignmentArm);
     }
 
     uint32_t dimensions = 0;
-    if (x > 1) dimensions++;
-    if (y > 1) dimensions++;
-    if (z > 1) dimensions++;
+    if (local_size.x > 1) dimensions++;
+    if (local_size.y > 1) dimensions++;
+    if (local_size.z > 1) dimensions++;
 
     if (dimensions == 1) {
         // If we're accessing images, we almost certainly want to have a 2D workgroup for cache reasons.
@@ -343,11 +312,10 @@ bool BestPractices::ValidateComputeShaderArm(const spirv::Module& module_state,
         for (const auto& variable : entrypoint.resource_interface_variables) {
             if (variable.IsImage() && variable.info.image_dim != spv::Dim1D && variable.info.image_dim != spv::DimBuffer) {
                 LogPerformanceWarning("BestPractices-Arm-vkCreateComputePipelines-compute-spatial-locality", device, loc,
-                                      "%s compute shader has work group dimensions (%" PRIu32 ", %" PRIu32 ", %" PRIu32
-                                      "), which "
+                                      "%s compute shader has work group dimensions (%s), which "
                                       "suggests a 1D dispatch, but the shader is accessing 2D or 3D images. The shader may be "
                                       "exhibiting poor spatial locality with respect to one or more shader resources.",
-                                      VendorSpecificTag(kBPVendorArm), x, y, z);
+                                      VendorSpecificTag(kBPVendorArm), local_size.ToString().c_str());
                 break;  // only need to report once
             }
         }
@@ -360,113 +328,24 @@ bool BestPractices::ValidateComputeShaderAmd(const spirv::Module& module_state,
                                              const Location& loc) const {
     bool skip = false;
 
-    uint32_t x = {}, y = {}, z = {};
-    if (!module_state.FindLocalSize(entrypoint, x, y, z)) {
+    spirv::LocalSize local_size = module_state.FindLocalSize(entrypoint);
+    if (local_size.x == 0) {
         return false;
     }
 
-    const uint64_t thread_count = x * y * z;
+    const uint64_t thread_count = local_size.x * local_size.y * local_size.z;
 
-    const bool multiple_64 = ((thread_count % 64) == 0);
-
-    if (!multiple_64) {
+    if (!IsIntegerMultipleOf(thread_count, 64)) {
         skip |= LogPerformanceWarning("BestPractices-AMD-LocalWorkgroup-Multiple64", device, loc,
-                                      "%s compute shader with work group dimensions (%" PRIu32 ", %" PRIu32 ", %" PRIu32
-                                      "), workgroup size (%" PRIu64
+                                      "%s compute shader with work group dimensions (%s), workgroup size (%" PRIu64
                                       "), is not a multiple of 64. Make the workgroup size a multiple of 64 to obtain best "
                                       "performance across all AMD GPU generations.",
-                                      VendorSpecificTag(kBPVendorAMD), x, y, z, thread_count);
+                                      VendorSpecificTag(kBPVendorAMD), local_size.ToString().c_str(), thread_count);
     }
 
     return skip;
 }
 
-void BestPractices::PostCallRecordCmdBindPipeline(VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint,
-                                                  VkPipeline pipeline, const RecordObject& record_obj) {
-    // AMD best practice
-    PipelineUsedInFrame(pipeline);
-
-    if (pipelineBindPoint == VK_PIPELINE_BIND_POINT_GRAPHICS) {
-        // check for depth/blend state tracking
-        if (auto pipeline_state = Get<vvl::Pipeline>(pipeline)) {
-            auto cb_state = GetWrite<vvl::CommandBuffer>(commandBuffer);
-            auto& sub_state = bp_state::SubState(*cb_state);
-            auto& render_pass_state = sub_state.render_pass_state;
-
-            render_pass_state.nextDrawTouchesAttachments = GetAttachmentAccess(*pipeline_state);
-            render_pass_state.drawTouchAttachments = true;
-
-            const auto* blend_state = pipeline_state->ColorBlendState();
-            const auto* stencil_state = pipeline_state->DepthStencilState();
-
-            if (blend_state && !(pipeline_state->ignore_color_attachments)) {
-                // assume the pipeline is depth-only unless any of the attachments have color writes enabled
-                render_pass_state.depthOnly = true;
-                for (size_t i = 0; i < blend_state->attachmentCount; i++) {
-                    if (blend_state->pAttachments[i].colorWriteMask != 0) {
-                        render_pass_state.depthOnly = false;
-                    }
-                }
-            }
-
-            // check for depth value usage
-            render_pass_state.depthEqualComparison = false;
-
-            if (stencil_state && stencil_state->depthTestEnable) {
-                switch (stencil_state->depthCompareOp) {
-                    case VK_COMPARE_OP_EQUAL:
-                    case VK_COMPARE_OP_GREATER_OR_EQUAL:
-                    case VK_COMPARE_OP_LESS_OR_EQUAL:
-                        render_pass_state.depthEqualComparison = true;
-                        break;
-                    default:
-                        break;
-                }
-            }
-
-            if (VendorCheckEnabled(kBPVendorNVIDIA)) {
-                using TessGeometryMeshState = bp_state::CommandBufferStateNV::TessGeometryMesh::State;
-                auto& tgm = sub_state.nv.tess_geometry_mesh;
-
-                // Make sure the message is only signaled once per command buffer
-                tgm.threshold_signaled = tgm.num_switches >= kNumBindPipelineTessGeometryMeshSwitchesThresholdNVIDIA;
-
-                // Track pipeline switches with tessellation, geometry, and/or mesh shaders enabled, and disabled
-                auto tgm_stages = VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT | VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT |
-                                  VK_SHADER_STAGE_GEOMETRY_BIT | VK_SHADER_STAGE_TASK_BIT_EXT | VK_SHADER_STAGE_MESH_BIT_EXT;
-                auto new_tgm_state = (pipeline_state->active_shaders & tgm_stages) != 0 ? TessGeometryMeshState::Enabled
-                                                                                        : TessGeometryMeshState::Disabled;
-                if (tgm.state != new_tgm_state && tgm.state != TessGeometryMeshState::Unknown) {
-                    tgm.num_switches++;
-                }
-                tgm.state = new_tgm_state;
-
-                // Track depthTestEnable and depthCompareOp
-                auto& pipeline_create_info = pipeline_state->GraphicsCreateInfo();
-                auto depth_stencil_state = pipeline_create_info.pDepthStencilState;
-                auto dynamic_state = pipeline_create_info.pDynamicState;
-                if (depth_stencil_state && dynamic_state) {
-                    auto dynamic_state_begin = dynamic_state->pDynamicStates;
-                    auto dynamic_state_end = dynamic_state->pDynamicStates + dynamic_state->dynamicStateCount;
-
-                    const bool dynamic_depth_test_enable =
-                        std::find(dynamic_state_begin, dynamic_state_end, VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE) != dynamic_state_end;
-                    const bool dynamic_depth_func =
-                        std::find(dynamic_state_begin, dynamic_state_end, VK_DYNAMIC_STATE_DEPTH_COMPARE_OP) != dynamic_state_end;
-
-                    if (!dynamic_depth_test_enable) {
-                        RecordSetDepthTestState(sub_state, sub_state.nv.depth_compare_op,
-                                                depth_stencil_state->depthTestEnable != VK_FALSE);
-                    }
-                    if (!dynamic_depth_func) {
-                        RecordSetDepthTestState(sub_state, depth_stencil_state->depthCompareOp, sub_state.nv.depth_test_enable);
-                    }
-                }
-            }
-        }
-    }
-}
-
 void BestPractices::PreCallRecordCreateGraphicsPipelines(VkDevice device, VkPipelineCache pipelineCache, uint32_t createInfoCount,
                                                          const VkGraphicsPipelineCreateInfo* pCreateInfos,
                                                          const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines,
@@ -498,7 +377,7 @@ bool BestPractices::PreCallValidateCreatePipelineLayout(VkDevice device, const V
 
         if (pipeline_size > kPipelineLayoutSizeWarningLimitAMD) {
             skip |= LogPerformanceWarning("BestPractices-AMD-CreatePipelinesLayout-KeepLayoutSmall", device, error_obj.location,
-                                          "%s pipeline layout size is too large. Prefer smaller pipeline layouts."
+                                          "%s pipeline layout size is too large. Prefer smaller pipeline layouts. "
                                           "Descriptor sets cost 1 DWORD each. "
                                           "Dynamic buffers cost 2 DWORDs each when robust buffer access is OFF. "
                                           "Dynamic buffers cost 4 DWORDs each when robust buffer access is ON. "
@@ -537,6 +416,7 @@ bool BestPractices::PreCallValidateCreatePipelineLayout(VkDevice device, const V
                         case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
                         case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR:
                         case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV:
+                        case VK_DESCRIPTOR_TYPE_PARTITIONED_ACCELERATION_STRUCTURE_NV:
                             descriptor_type_size = 8;
                             break;
                         case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER:
@@ -586,25 +466,27 @@ bool BestPractices::PreCallValidateCmdBindPipeline(VkCommandBuffer commandBuffer
                                                    VkPipeline pipeline, const ErrorObject& error_obj) const {
     bool skip = false;
 
-    if (VendorCheckEnabled(kBPVendorAMD) || VendorCheckEnabled(kBPVendorNVIDIA)) {
-        if (IsPipelineUsedInFrame(pipeline)) {
-            skip |= LogPerformanceWarning(
-                "BestPractices-Pipeline-SortAndBind", commandBuffer, error_obj.location,
-                "%s %s Pipeline %s was bound twice in the frame. "
-                "Keep pipeline state changes to a minimum, for example, by sorting draw calls by pipeline.",
-                VendorSpecificTag(kBPVendorAMD), VendorSpecificTag(kBPVendorNVIDIA), FormatHandle(pipeline).c_str());
+    if (pipelineBindPoint == VK_PIPELINE_BIND_POINT_GRAPHICS) {
+        if (VendorCheckEnabled(kBPVendorAMD) || VendorCheckEnabled(kBPVendorNVIDIA)) {
+            if (IsPipelineUsedInFrame(pipeline)) {
+                skip |= LogPerformanceWarning(
+                    "BestPractices-Pipeline-SortAndBind", commandBuffer, error_obj.location,
+                    "%s %s Pipeline %s was bound twice in the frame. "
+                    "Keep pipeline state changes to a minimum, for example, by sorting draw calls by pipeline.",
+                    VendorSpecificTag(kBPVendorAMD), VendorSpecificTag(kBPVendorNVIDIA), FormatHandle(pipeline).c_str());
+            }
         }
-    }
-    if (VendorCheckEnabled(kBPVendorNVIDIA)) {
-        auto cb_state = Get<vvl::CommandBuffer>(commandBuffer);
-        auto& sub_state = bp_state::SubState(*cb_state);
-        const auto& tgm = sub_state.nv.tess_geometry_mesh;
-        if (tgm.num_switches >= kNumBindPipelineTessGeometryMeshSwitchesThresholdNVIDIA && !tgm.threshold_signaled) {
-            LogPerformanceWarning("BestPractices-NVIDIA-BindPipeline-SwitchTessGeometryMesh", commandBuffer, error_obj.location,
-                                  "%s Avoid switching between pipelines with and without tessellation, geometry, task, "
-                                  "and/or mesh shaders. Group draw calls using these shader stages together.",
-                                  VendorSpecificTag(kBPVendorNVIDIA));
-            // Do not set 'skip' so the number of switches gets properly counted after the message.
+        if (VendorCheckEnabled(kBPVendorNVIDIA)) {
+            auto cb_state = Get<vvl::CommandBuffer>(commandBuffer);
+            auto& sub_state = bp_state::SubState(*cb_state);
+            const auto& tgm = sub_state.nv.tess_geometry_mesh;
+            if (tgm.num_switches >= kNumBindPipelineTessGeometryMeshSwitchesThresholdNVIDIA && !tgm.threshold_signaled) {
+                LogPerformanceWarning("BestPractices-NVIDIA-BindPipeline-SwitchTessGeometryMesh", commandBuffer, error_obj.location,
+                                      "%s Avoid switching between pipelines with and without tessellation, geometry, task, "
+                                      "and/or mesh shaders. Group draw calls using these shader stages together.",
+                                      VendorSpecificTag(kBPVendorNVIDIA));
+                // Do not set 'skip' so the number of switches gets properly counted after the message.
+            }
         }
     }
 
@@ -638,7 +520,7 @@ bool BestPractices::ValidateShaderStage(const ShaderStageState& stage_state, con
         }
 
         if (IsExtEnabled(extensions.vk_khr_maintenance4)) {
-            if (module_state.static_data_.has_builtin_workgroup_size) {
+            if (module_state.static_data_.has_built_in_workgroup_size) {
                 skip |= LogWarning("BestPractices-SpirvDeprecated_WorkgroupSize", device, loc,
                                    "is using the SPIR-V Workgroup built-in which SPIR-V 1.6 deprecated. When using "
                                    "VK_KHR_maintenance4 or Vulkan 1.3+, the new SPIR-V LocalSizeId execution mode should be used "
@@ -647,5 +529,27 @@ bool BestPractices::ValidateShaderStage(const ShaderStageState& stage_state, con
         }
     }
 
+    if (entrypoint.stage == VK_SHADER_STAGE_TASK_BIT_EXT || entrypoint.stage == VK_SHADER_STAGE_MESH_BIT_EXT) {
+        spirv::LocalSize local_size = module_state.FindLocalSize(entrypoint);
+        if (local_size.x == 0) {
+            return skip;
+        }
+        bool is_task =
+            entrypoint.execution_model == spv::ExecutionModelTaskEXT || entrypoint.execution_model == spv::ExecutionModelTaskNV;
+
+        // Assume core checks caught any overflow
+        uint64_t invocations =
+            static_cast<uint64_t>(local_size.x) * static_cast<uint64_t>(local_size.y) * static_cast<uint64_t>(local_size.z);
+        const uint32_t preferred_size = is_task ? phys_dev_ext_props.mesh_shader_props_ext.maxPreferredTaskWorkGroupInvocations
+                                                : phys_dev_ext_props.mesh_shader_props_ext.maxPreferredMeshWorkGroupInvocations;
+        if (invocations > preferred_size) {
+            skip |= LogPerformanceWarning(
+                "BestPractices-Mesh-MaxPreferredWorkGroupInvocations", module_state.handle(), loc,
+                "SPIR-V (%s) total invocation size of %" PRIu64 " (%s) is more than %s (%" PRIu32 ").",
+                string_SpvExecutionModel(entrypoint.execution_model), invocations, local_size.ToString().c_str(),
+                is_task ? "maxPreferredTaskWorkGroupInvocations" : "maxPreferredMeshWorkGroupInvocations", preferred_size);
+        }
+    }
+
     return skip;
 }
\ No newline at end of file
