@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2025 The Khronos Group Inc.
- * Copyright (c) 2015-2025 Valve Corporation
- * Copyright (c) 2015-2025 LunarG, Inc.
- * Copyright (C) 2015-2025 Google Inc.
+/* Copyright (c) 2015-2026 The Khronos Group Inc.
+ * Copyright (c) 2015-2026 Valve Corporation
+ * Copyright (c) 2015-2026 LunarG, Inc.
+ * Copyright (C) 2015-2026 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,24 +17,29 @@
  * limitations under the License.
  */
 
+#include <spirv/unified1/spirv.hpp>
 #include <cassert>
 #include <sstream>
 #include <string>
 #include <vector>
 
 #include <vulkan/vk_enum_string_helper.h>
+#include <vulkan/utility/vk_format_utils.h>
 #include <vulkan/vulkan_core.h>
 #include "core_validation.h"
 #include "generated/spirv_grammar_helper.h"
 #include "state_tracker/image_state.h"
+#include "state_tracker/shader_instruction.h"
 #include "state_tracker/shader_object_state.h"
 #include "state_tracker/shader_stage_state.h"
 #include "state_tracker/shader_module.h"
 #include "state_tracker/render_pass_state.h"
 #include "state_tracker/cmd_buffer_state.h"
 #include "state_tracker/pipeline_state.h"
-#include "utils/vk_layer_utils.h"
 #include "containers/limits.h"
+#include "error_message/error_strings.h"
+#include "utils/vk_api_utils.h"
+#include "utils/image_utils.h"
 
 bool CoreChecks::ValidateInterfaceVertexInput(const vvl::Pipeline &pipeline, const spirv::Module &module_state,
                                               const spirv::EntryPoint &entrypoint, const Location &create_info_loc) const {
@@ -88,7 +93,8 @@ bool CoreChecks::ValidateInterfaceVertexInput(const vvl::Pipeline &pipeline, con
             for (uint32_t i = 0; i < variable.type_struct_info->members.size(); i++) {
                 const auto &member = variable.type_struct_info->members[i];
                 // can be 64-bit formats in the struct
-                const uint32_t num_locations = module_state.GetLocationsConsumedByType(member.id);
+                const spirv::Instruction* member_type = module_state.FindDef(member.id);
+                const uint32_t num_locations = module_state.GetLocationsConsumedByType(member_type);
                 for (uint32_t j = 0; j < num_locations; ++j) {
                     location_map[location + j].shader_input = member.insn;
                 }
@@ -110,22 +116,23 @@ bool CoreChecks::ValidateInterfaceVertexInput(const vvl::Pipeline &pipeline, con
             skip |= LogPerformanceWarning("WARNING-Shader-OutputNotConsumed", module_state.handle(), vi_loc,
                                           "Vertex attribute at location %" PRIu32 " not consumed by vertex shader.", location);
         } else if (!attribute_input && shader_input) {
-            if (!enabled_features.vertexAttributeRobustness) {
+            if (!enabled_features.vertexAttributeRobustness && !enabled_features.maintenance9) {
                 skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-Input-07904", module_state.handle(),
                                  vi_loc.dot(Field::pVertexAttributeDescriptions),
                                  "does not have a Location %" PRIu32
                                  " but vertex shader has an input variable at that Location. (This can be valid if "
-                                 "vertexAttributeRobustness feature is enabled)",
+                                 "either the vertexAttributeRobustness or maintenance9 feature is enabled)",
                                  location);
             }
         } else if (attribute_input && shader_input) {
             const VkFormat attribute_format = *attribute_input;
             const uint32_t attribute_type = spirv::GetFormatType(attribute_format);
             const uint32_t var_base_type_id = shader_input->ResultId();
+            const spirv::Instruction* var_base_type = module_state.FindDef(var_base_type_id);
             const uint32_t var_numeric_type = module_state.GetNumericType(var_base_type_id);
 
             const bool attribute64 = vkuFormatIs64bit(attribute_format);
-            const bool shader64 = module_state.GetBaseTypeInstruction(var_base_type_id)->GetBitWidth() == 64;
+            const bool shader64 = module_state.GetBaseTypeInstruction(var_base_type)->GetBitWidth() == 64;
 
             // Type checking
             if ((attribute_type & var_numeric_type) == 0) {
@@ -182,7 +189,7 @@ bool CoreChecks::ValidateInterfaceFragmentOutput(const vvl::Pipeline &pipeline,
     return skip;
 }
 
-bool CoreChecks::ValidateBuiltinLimits(const spirv::Module &module_state, const spirv::EntryPoint &entrypoint,
+bool CoreChecks::ValidateBuiltInLimits(const spirv::Module &module_state, const spirv::EntryPoint &entrypoint,
                                        const vvl::Pipeline *pipeline, const Location &loc) const {
     bool skip = false;
 
@@ -194,7 +201,7 @@ bool CoreChecks::ValidateBuiltinLimits(const spirv::Module &module_state, const
     for (const auto *variable : entrypoint.built_in_variables) {
         // Currently don't need to search in structs
         // Handles both the input and output sampleMask
-        if (variable->decorations.builtin == spv::BuiltInSampleMask &&
+        if (variable->decorations.built_in == spv::BuiltInSampleMask &&
             variable->array_size > phys_dev_props.limits.maxSampleMaskWords) {
             const char *vuid = pipeline ? "VUID-VkPipelineShaderStageCreateInfo-maxSampleMaskWords-00711"
                                         : "VUID-VkShaderCreateInfoEXT-pCode-08451";
@@ -220,34 +227,38 @@ bool CoreChecks::ValidatePrimitiveTopology(const spirv::Module &module_state, co
     }
 
     bool has_tess = false;
-    VkPrimitiveTopology topology = pipeline.InputAssemblyState()->topology;
+    VkPrimitiveTopology tess_output_topology = VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
     for (uint32_t i = 0; i < pipeline.stage_states.size(); i++) {
         auto &stage_state = pipeline.stage_states[i];
         const VkShaderStageFlagBits stage = stage_state.GetStage();
         if (stage == VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT || stage == VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT) {
             has_tess = true;
+            // PointMode will take precedence over everything
+            // (details in https://gitlab.khronos.org/vulkan/vulkan/-/merge_requests/7404)
             if (stage_state.entrypoint->execution_mode.Has(spirv::ExecutionModeSet::point_mode_bit)) {
-                topology = VK_PRIMITIVE_TOPOLOGY_POINT_LIST;
-            } else {
-                topology = stage_state.entrypoint->execution_mode.primitive_topology;
+                tess_output_topology = VK_PRIMITIVE_TOPOLOGY_POINT_LIST;
+                break;
+            } else if (stage == VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT) {
+                tess_output_topology = stage_state.entrypoint->execution_mode.GetTessellationEvalOutputTopology();
             }
         }
     }
 
-    VkPrimitiveTopology geom_topology = entrypoint.execution_mode.input_primitive_topology;
-    bool mismatch = false;
-    mismatch |= (topology == VK_PRIMITIVE_TOPOLOGY_POINT_LIST && geom_topology != VK_PRIMITIVE_TOPOLOGY_POINT_LIST);
-    mismatch |= IsLineTopology(topology) && !IsLineTopology(geom_topology);
-    mismatch |= IsTriangleTopology(topology) && !IsTriangleTopology(geom_topology);
-    if (mismatch) {
-        if (has_tess) {
+    const VkPrimitiveTopology geom_input_topology = entrypoint.execution_mode.GetGeometryInputTopology();
+    if (has_tess) {
+        if (!IsSameTopologyClass(geom_input_topology, tess_output_topology)) {
             skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-pStages-00739", module_state.handle(), loc,
-                             "SPIR-V (Geometry stage) expects input topology %s, but tessellation shader output topology is %s.",
-                             string_VkPrimitiveTopology(geom_topology), string_VkPrimitiveTopology(topology));
-        } else {
+                             "SPIR-V (Geometry stage) has declares in the shader input topology %s, but tessellation shader output "
+                             "topology is %s.",
+                             string_VkPrimitiveTopology(geom_input_topology), string_VkPrimitiveTopology(tess_output_topology));
+        }
+    } else {
+        VkPrimitiveTopology vertex_input_topology = pipeline.InputAssemblyState()->topology;
+        if (!IsSameTopologyClass(geom_input_topology, vertex_input_topology)) {
             skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-pStages-00738", module_state.handle(), loc,
-                             "SPIR-V (Geometry stage) expects input topology %s, but pipeline was created with primitive topology %s.",
-                             string_VkPrimitiveTopology(geom_topology), string_VkPrimitiveTopology(topology));
+                             "SPIR-V (Geometry stage) has declares in the shader input topology %s, but pipeline was created with "
+                             "a vertex input primitive topology of %s.",
+                             string_VkPrimitiveTopology(geom_input_topology), string_VkPrimitiveTopology(vertex_input_topology));
         }
     }
 
@@ -344,35 +355,61 @@ bool CoreChecks::ValidateInterfaceBetweenStages(const spirv::Module &producer, c
                 // }
 
                 // If using maintenance4 need to check Vectors incase different sizes
-                if (!enabled_features.maintenance4 && (output_var->base_type.Opcode() == spv::OpTypeVector) &&
-                    (input_var->base_type.Opcode() == spv::OpTypeVector)) {
+                if (!enabled_features.maintenance4 && output_var->base_type.IsVector() && input_var->base_type.IsVector()) {
                     // Note the "Component Count" in the VU refers to OpTypeVector's operand and NOT the "Component slot"
-                    const uint32_t output_vec_size = output_var->base_type.Word(3);
-                    const uint32_t input_vec_size = input_var->base_type.Word(3);
+                    const uint32_t output_vec_size =
+                        producer.GetNumComponentsInBaseType(producer.FindDef(output_var->base_type.ResultId()));
+                    const uint32_t input_vec_size =
+                        consumer.GetNumComponentsInBaseType(consumer.FindDef(input_var->base_type.ResultId()));
                     if (output_vec_size > input_vec_size) {
                         const LogObjectList objlist(producer.handle(), consumer.handle());
                         skip |= LogError("VUID-RuntimeSpirv-maintenance4-06817", objlist, create_info_loc,
                                          "(SPIR-V Interface) starting at Location %" PRIu32 " Component %" PRIu32
-                                         " the Output (%s) has a Vec%" PRIu32 " while Input (%s) as a Vec%" PRIu32
-                                         ". Enable VK_KHR_maintenance4 device extension to allow relaxed interface matching "
-                                         "between input and output vectors.",
+                                         "\nThe output (%s) has a Vec%" PRIu32 "\nThe input (%s) has a Vec%" PRIu32
+                                         "\nThis mismatch between vector sizes is not allowed, but can be relaxed by enabling "
+                                         "VK_KHR_maintenance4.",
                                          location, component, string_VkShaderStageFlagBits(producer_stage), output_vec_size,
                                          string_VkShaderStageFlagBits(consumer_stage), input_vec_size);
                         break;  // Only need to report for the first component found
                     }
                 }
+
+                if (producer_stage == VK_SHADER_STAGE_MESH_BIT_EXT) {
+                    if (input_var->is_per_primitive_ext != output_var->is_per_primitive_ext) {
+                        const LogObjectList objlist(producer.handle(), consumer.handle());
+                        std::ostringstream ss;
+                        ss << "(SPIR-V Interface) at Location " << location << " Component " << component
+                           << " in the Mesh stage is " << (output_var->is_per_primitive_ext ? "" : "not ")
+                           << "decorated with PerPrimitiveEXT while the Fragment stage is "
+                           << (input_var->is_per_primitive_ext ? "" : "not") << ".";
+                        if (consumer.static_data_.source_language == spv::SourceLanguageGLSL) {
+                            ss << "\nMake sure to use the 'perprimitiveEXT' attribute on your interface variables. The '#extension "
+                                  "GL_EXT_mesh_shader' is also required, even in the fragment shader.";
+                        } else if (consumer.static_data_.source_language == spv::SourceLanguageHLSL) {
+                            ss << "\nThis currently is a known limitation in HLSL, but has a workaroud, see "
+                                  "https://github.com/microsoft/DirectXShaderCompiler/issues/6862";
+                        } else if (consumer.static_data_.source_language == spv::SourceLanguageSlang) {
+                            ss << "\nThis currently is a known limitation in Slang, see "
+                                  "https://github.com/shader-slang/slang/issues/7019";
+                        }
+                        skip |= LogError("VUID-RuntimeSpirv-OpVariable-08746", objlist, create_info_loc, "%s", ss.str().c_str());
+                        break;  // Only need to report for the first component found
+                    }
+                }
+
             } else if ((input_var == nullptr) && (output_var != nullptr)) {
                 // Missing input slot
                 // It is not an error if a stage does not consume all outputs from the previous stage
-                // The values will be undefined, but still legal
                 // Don't give any warning if maintenance4 with vectors
-                if (!enabled_features.maintenance4 && (output_var->base_type.Opcode() != spv::OpTypeVector)) {
+                if (!enabled_features.maintenance4 && !output_var->base_type.IsVector()) {
                     const LogObjectList objlist(producer.handle(), consumer.handle());
-                    skip |= LogPerformanceWarning("WARNING-Shader-OutputNotConsumed", objlist, create_info_loc,
-                                                  "(SPIR-V Interface) %s declared to output location %" PRIu32 " Component %" PRIu32
-                                                  " but is not an Input declared by %s.",
-                                                  string_VkShaderStageFlagBits(producer_stage), location, component,
-                                                  string_VkShaderStageFlagBits(consumer_stage));
+                    skip |= LogPerformanceWarning(
+                        "WARNING-Shader-OutputNotConsumed", objlist, create_info_loc,
+                        "(SPIR-V Interface) %s has an Output value declared at Location %" PRIu32 " Component %" PRIu32
+                        ", but there is no corresponding Input declared in %s.\nThis is not invalid, but might the write to to the "
+                        "unused Output is discarded.\nThe Output variable is:\n  %s",
+                        string_VkShaderStageFlagBits(producer_stage), location, component,
+                        string_VkShaderStageFlagBits(consumer_stage), producer.DescribeType(output_var->type_id).c_str());
                 }
             } else if ((input_var != nullptr) && (output_var == nullptr)) {
                 // Missing output slot
@@ -383,10 +420,11 @@ bool CoreChecks::ValidateInterfaceBetweenStages(const spirv::Module &producer, c
                 }
                 const LogObjectList objlist(producer.handle(), consumer.handle());
                 skip |= LogError("VUID-RuntimeSpirv-OpEntryPoint-08743", objlist, create_info_loc,
-                                 "(SPIR-V Interface) %s declared input at Location %" PRIu32 " Component %" PRIu32
-                                 " %sbut it is not an Output declared in %s",
+                                 "(SPIR-V Interface) %s has a declared Input at Location %" PRIu32 " Component %" PRIu32
+                                 " %s but the previous stage (%s) has no Output declared there.\nThe input variable is:\n  %s",
                                  string_VkShaderStageFlagBits(consumer_stage), location, component,
-                                 input_var->is_patch ? "(Tessellation Patch) " : "", string_VkShaderStageFlagBits(producer_stage));
+                                 input_var->is_patch ? "(Tessellation Patch) " : "", string_VkShaderStageFlagBits(producer_stage),
+                                 consumer.DescribeType(input_var->type_id).c_str());
                 break;  // Only need to report for the first component found
             }
         }
@@ -397,49 +435,49 @@ bool CoreChecks::ValidateInterfaceBetweenStages(const spirv::Module &producer, c
         return skip;
     }
 
-    std::vector<uint32_t> input_builtins_block;
-    std::vector<uint32_t> output_builtins_block;
+    std::vector<spv::BuiltIn> input_built_in_block;
+    std::vector<spv::BuiltIn> output_built_in_block;
     for (const auto *variable : producer_entrypoint.built_in_variables) {
-        if (variable->storage_class == spv::StorageClassOutput && !variable->builtin_block.empty()) {
-            output_builtins_block = variable->builtin_block;
+        if (variable->storage_class == spv::StorageClassOutput && !variable->built_in_block.empty()) {
+            output_built_in_block = variable->built_in_block;
             break;
         }
     }
     for (const auto *variable : consumer_entrypoint.built_in_variables) {
-        if (variable->storage_class == spv::StorageClassInput && !variable->builtin_block.empty()) {
-            input_builtins_block = variable->builtin_block;
+        if (variable->storage_class == spv::StorageClassInput && !variable->built_in_block.empty()) {
+            input_built_in_block = variable->built_in_block;
             break;
         }
     }
 
     bool mismatch = false;
-    if (input_builtins_block.empty() || output_builtins_block.empty()) {
+    if (input_built_in_block.empty() || output_built_in_block.empty()) {
         // TODO - Nothing about this in spec, need to add language to confirm this is correct
         return skip;
-    } else if (input_builtins_block.size() != output_builtins_block.size()) {
+    } else if (input_built_in_block.size() != output_built_in_block.size()) {
         mismatch = true;
     } else {
-        for (size_t i = 0; i < input_builtins_block.size(); i++) {
-            const uint32_t input_builtin = input_builtins_block[i];
-            const uint32_t output_builtin = output_builtins_block[i];
-            if (input_builtin == spirv::kInvalidValue || output_builtin == spirv::kInvalidValue) {
+        for (size_t i = 0; i < input_built_in_block.size(); i++) {
+            const spv::BuiltIn input_built_in = input_built_in_block[i];
+            const spv::BuiltIn output_built_in = output_built_in_block[i];
+            if (input_built_in == spirv::kInvalidBuiltIn || output_built_in == spirv::kInvalidBuiltIn) {
                 continue;  // some stages (TessControl -> TessEval) can have legal block vs non-block mismatch
-            } else if (input_builtin != output_builtin) {
+            } else if (input_built_in != output_built_in) {
                 mismatch = true;
             }
         }
     }
 
     if (mismatch) {
-        std::stringstream msg;
+        std::ostringstream msg;
         msg << string_VkShaderStageFlagBits(producer_stage) << " Output Block {\n";
-        for (size_t i = 0; i < output_builtins_block.size(); i++) {
-            msg << '\t' << i << ": " << string_SpvBuiltIn(output_builtins_block[i]) << '\n';
+        for (size_t i = 0; i < output_built_in_block.size(); i++) {
+            msg << '\t' << i << ": " << string_SpvBuiltIn(output_built_in_block[i]) << '\n';
         }
         msg << "}\n";
         msg << string_VkShaderStageFlagBits(consumer_stage) << " Input Block {\n";
-        for (size_t i = 0; i < input_builtins_block.size(); i++) {
-            msg << '\t' << i << ": " << string_SpvBuiltIn(input_builtins_block[i]) << '\n';
+        for (size_t i = 0; i < input_built_in_block.size(); i++) {
+            msg << '\t' << i << ": " << string_SpvBuiltIn(input_built_in_block[i]) << '\n';
         }
         msg << "}\n";
         const LogObjectList objlist(producer.handle(), consumer.handle());
@@ -455,6 +493,10 @@ bool CoreChecks::ValidateFsOutputsAgainstRenderPass(const spirv::Module &module_
                                                     const vvl::Pipeline &pipeline, uint32_t subpass_index,
                                                     const Location &create_info_loc) const {
     bool skip = false;
+    // To match with the dynamic rendering case
+    if (global_settings.only_report_errors) {
+        return skip;
+    }
 
     struct Attachment {
         const VkAttachmentReference2 *reference = nullptr;
@@ -542,25 +584,51 @@ bool CoreChecks::ValidateFsOutputsAgainstRenderPass(const spirv::Module &module_
     return skip;
 }
 
+static std::string DescribeMappedLocation(uint32_t shader, uint32_t rendering_info) {
+    std::ostringstream msg;
+    if (shader == rendering_info) {
+        msg << shader;
+    } else {
+        msg << shader << " (which was remapped to attachment " << rendering_info << ")";
+    }
+    return msg.str();
+}
+
 // This is validated at draw time unlike the VkRenderPass version
-bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bound_state, const vvl::Pipeline *pipeline,
-                                                       const vvl::RenderPass &rp_state, const Location &loc) const {
+bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bound_state, const vvl::CommandBuffer &cb_state,
+                                                       const Location &loc) const {
     bool skip = false;
-
+    // Some apps do 100k draws a frame and found this function is a bottleneck if an app is violating these warnings
+    if (global_settings.only_report_errors) {
+        return skip;
+    }
+    if (last_bound_state.IsRasterizationDisabled()) {
+        return skip;
+    }
     const spirv::EntryPoint *entrypoint = last_bound_state.GetFragmentEntryPoint();
-    if (!entrypoint) return skip;
-
-    if (rp_state.use_dynamic_rendering_inherited) return skip;
+    if (!entrypoint) {
+        return skip;
+    }
+    const vvl::RenderPass &rp_state = *cb_state.active_render_pass;
+    if (rp_state.use_dynamic_rendering_inherited) {
+        return skip;
+    }
+    vvl::Pipeline *pipeline = last_bound_state.pipeline_state;
 
     struct Attachment {
         const VkRenderingAttachmentInfo *rendering_attachment_info = nullptr;
         const spirv::StageInterfaceVariable *output = nullptr;
+        std::optional<uint32_t> mapped_location = std::nullopt;
     };
     std::map<uint32_t, Attachment> location_map;
 
-    const uint32_t color_attachment_count = rp_state.dynamic_rendering_begin_rendering_info.colorAttachmentCount;
-    for (uint32_t i = 0; i < color_attachment_count; ++i) {
-        location_map[i].rendering_attachment_info = rp_state.dynamic_rendering_begin_rendering_info.pColorAttachments[i].ptr();
+    const size_t color_attachment_count = cb_state.rendering_attachments.color_locations.size();
+    for (size_t i = 0; i < color_attachment_count; ++i) {
+        uint32_t color_location = cb_state.rendering_attachments.color_locations[i];
+        if (color_location != VK_ATTACHMENT_UNUSED) {
+            location_map[color_location].rendering_attachment_info = rp_state.dynamic_rendering_begin_rendering_info.pColorAttachments[i].ptr();
+            location_map[color_location].mapped_location = static_cast<uint32_t>(i);
+        }
     }
 
     // TODO: dual source blend index (spv::DecIndex, zero if not provided)
@@ -583,6 +651,7 @@ bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bou
         const bool has_attachment =
             attachment_info.rendering_attachment_info && attachment_info.rendering_attachment_info->imageView != VK_NULL_HANDLE;
         const spirv::StageInterfaceVariable *output = attachment_info.output;
+        uint32_t mapped_loc = attachment_info.mapped_location.value_or(location);
 
         if (has_attachment && !output) {
             // https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/9616
@@ -593,21 +662,29 @@ bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bou
                 const bool null_image_view = attachment_info.rendering_attachment_info &&
                                              attachment_info.rendering_attachment_info->imageView == VK_NULL_HANDLE;
                 const LogObjectList objlist = last_bound_state.cb_state.GetObjectList(VK_PIPELINE_BIND_POINT_GRAPHICS);
-                skip |= LogUndefinedValue("Undefined-Value-ShaderOutputNotConsumed-DynamicRendering", objlist, loc,
-                                          "Inside the fragment shader, it writes to output Location %" PRIu32
-                                          " but there is no VkRenderingInfo::pColorAttachments[%" PRIu32
-                                          "]%s and this write is unused.\nSpec information at "
-                                          "https://docs.vulkan.org/spec/latest/chapters/interfaces.html#interfaces-fragmentoutput",
-                                          location, location, null_image_view ? " (imageView is VK_NULL_HANDLE)" : "");
+                std::ostringstream reason;
+                if (null_image_view || location >= cb_state.rendering_attachments.color_locations.size()) {
+                    reason << "there is no VkRenderingInfo::pColorAttachments[" << mapped_loc << "]";
+                    if (null_image_view) {
+                        reason << " (imageView is VK_NULL_HANDLE)";
+                    }
+                } else {
+                    reason << "none of the attachments were mapped to that location (mapping was [";
+                    for (const uint32_t &mapping : cb_state.rendering_attachments.color_locations) {
+                        if (&mapping != &cb_state.rendering_attachments.color_locations[0]) {
+                            reason << ", ";
+                        }
+                        reason << string_Attachment(mapping);
+                    }
+                    reason << "])";
+                }
+                skip |= LogUndefinedValue(
+                    "Undefined-Value-ShaderOutputNotConsumed-DynamicRendering", objlist, loc,
+                    "Inside the fragment shader, it writes to output Location %s but %s and this write is unused.\n"
+                    "Spec information at https://docs.vulkan.org/spec/latest/chapters/interfaces.html#interfaces-fragmentoutput",
+                    DescribeMappedLocation(location, mapped_loc).c_str(), reason.str().c_str());
             }
         } else if (has_attachment && output) {
-            if (last_bound_state.cb_state.rendering_attachments.set_color_indexes ||
-                last_bound_state.cb_state.rendering_attachments.set_color_locations) {
-                // TODO - Handle VK_KHR_dynamic_rendering_local_read
-                // https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8887
-                continue;
-            }
-
             const auto image_view_state = Get<vvl::ImageView>(attachment_info.rendering_attachment_info->imageView);
             const uint32_t attachment_type = spirv::GetFormatType(image_view_state->create_info.format);
 
@@ -617,7 +694,7 @@ bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bou
                 pipeline->fragment_shader_state->fragment_shader->spirv) {
                 module_state = pipeline->fragment_shader_state->fragment_shader->spirv.get();
             } else if (!pipeline) {
-                const vvl::ShaderObject *shader_object = last_bound_state.GetShaderStateIfValid(ShaderObjectStage::FRAGMENT);
+                const vvl::ShaderObject *shader_object = last_bound_state.GetShaderObjectStateIfValid(ShaderObjectStage::FRAGMENT);
                 if (shader_object && shader_object->spirv) {
                     module_state = shader_object->spirv.get();
                 }
@@ -628,15 +705,15 @@ bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bou
             // Type checking
             if ((output_type & attachment_type) == 0) {
                 const LogObjectList objlist = last_bound_state.cb_state.GetObjectList(VK_PIPELINE_BIND_POINT_GRAPHICS);
-                skip |= LogUndefinedValue("Undefined-Value-ShaderFragmentOutputMismatch-DynamicRendering", objlist, loc,
-                                          "Inside the fragment shader, it writes to output Location %" PRIu32
-                                          " with a numeric type of %s but VkRenderingInfo::pColorAttachments[%" PRIu32
-                                          "].imageView is created with %s (numeric type of %s) which does not match and the "
-                                          "resulting values written will be undefined.\nSpec information at "
-                                          "https://docs.vulkan.org/spec/latest/chapters/interfaces.html#interfaces-fragmentoutput",
-                                          location, spirv::string_NumericType(output_type), location,
-                                          string_VkFormat(image_view_state->create_info.format),
-                                          spirv::string_NumericType(attachment_type));
+                skip |= LogUndefinedValue(
+                    "Undefined-Value-ShaderFragmentOutputMismatch-DynamicRendering", objlist, loc,
+                    "Inside the fragment shader, it writes to output Location %s with a numeric type of %s but "
+                    "VkRenderingInfo::pColorAttachments[%" PRIu32
+                    "].imageView is created with %s (numeric type of %s) which does not match and the "
+                    "resulting values written will be undefined.\n"
+                    "Spec information at https://docs.vulkan.org/spec/latest/chapters/interfaces.html#interfaces-fragmentoutput",
+                    DescribeMappedLocation(location, mapped_loc).c_str(), spirv::string_NumericType(output_type), mapped_loc,
+                    string_VkFormat(image_view_state->create_info.format), spirv::string_NumericType(attachment_type));
             }
         } else {  // !attachment && !output
             // Means empty fragment shader and no color attachments
@@ -647,6 +724,27 @@ bool CoreChecks::ValidateDrawDynamicRenderingFsOutputs(const LastBound &last_bou
     return skip;
 }
 
+bool CoreChecks::ValidateDrawRenderingTileMemoryOutputs(const LastBound &last_bound_state, const vvl::CommandBuffer &cb_state,
+                                                        const vvl::DrawDispatchVuid &vuid) const {
+    bool skip = false;
+
+    if (last_bound_state.IsRasterizationDisabled()) {
+        return skip;
+    }
+
+    for (uint32_t i = 0; i < cb_state.active_attachments.size(); ++i) {
+        const auto &attachment_info = cb_state.active_attachments[i];
+        const auto image_view_state = attachment_info.image_view;
+        // Resolve is banned with a separate VU and checked elsewhere
+        if (image_view_state && !attachment_info.IsResolve()) {
+            skip |= ValidateBoundTileMemory(*image_view_state->image_state, cb_state, vuid);
+
+        }
+    }
+
+    return skip;
+}
+
 bool CoreChecks::ValidatePipelineTessellationStages(const spirv::Module &tesc_module_state,
                                                     const spirv::EntryPoint &tesc_entrypoint,
                                                     const spirv::Module &tese_module_state,
@@ -654,30 +752,34 @@ bool CoreChecks::ValidatePipelineTessellationStages(const spirv::Module &tesc_mo
                                                     const Location &create_info_loc) const {
     bool skip = false;
 
-    const auto tesc_subdivision = tesc_entrypoint.execution_mode.tessellation_subdivision;
-    const auto tese_subdivision = tese_entrypoint.execution_mode.tessellation_subdivision;
-    const auto tesc_patch_size = tesc_entrypoint.execution_mode.output_vertices;
-    const auto tese_patch_size = tese_entrypoint.execution_mode.output_vertices;
-    if (tesc_subdivision == 0 && tese_subdivision == 0) {
+    // The other tessellation modes (PointMode, Spacing, Orientation) can be in either stage.
+    // OutputVertices needs to be in TESC and Subdivision in TESE, but they can be in the other stage, but needs to match
+    const uint32_t tesc_subdivision = tesc_entrypoint.execution_mode.GetTessellationSubdivision();
+    const uint32_t tese_subdivision = tese_entrypoint.execution_mode.GetTessellationSubdivision();
+    const uint32_t tesc_patch_size = tesc_entrypoint.execution_mode.output_vertices;
+    const uint32_t tese_patch_size = tese_entrypoint.execution_mode.output_vertices;
+    if (tesc_subdivision == spirv::kInvalidValue && tese_subdivision == spirv::kInvalidValue) {
         const LogObjectList objlist(tesc_module_state.handle(), tese_module_state.handle());
         skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-pStages-00732", objlist, create_info_loc,
-                         "Subdivision type is not specified in either of tessellation stages");
-    } else if (tesc_subdivision != 0 && tese_subdivision != 0 && tesc_subdivision != tese_subdivision) {
+                         "Subdivision (Triangles/Quads/IsoLines) is not specified in either of tessellation stages");
+    } else if (tesc_subdivision != spirv::kInvalidValue && tese_subdivision != spirv::kInvalidValue &&
+               tesc_subdivision != tese_subdivision) {
         const LogObjectList objlist(tesc_module_state.handle(), tese_module_state.handle());
         skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-pStages-00733", objlist, create_info_loc,
-                         "Subdivision type specified in tessellation control shader is %s, but subdivison type specified in "
+                         "Subdivision specified in tessellation control shader is %s, but subdivison type specified in "
                          "tessellation evaluation shader is %s",
                          string_SpvExecutionMode(tesc_subdivision), string_SpvExecutionMode(tese_subdivision));
     }
-    if (tesc_patch_size == vvl::kU32Max && tese_patch_size == vvl::kU32Max) {
+    if (tesc_patch_size == spirv::kInvalidValue && tese_patch_size == spirv::kInvalidValue) {
         const LogObjectList objlist(tesc_module_state.handle(), tese_module_state.handle());
         skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-pStages-00734", objlist, create_info_loc,
-                         "Output patch size is not specified in either of tessellation stages");
-    } else if (tesc_patch_size != vvl::kU32Max && tese_patch_size != vvl::kU32Max && tesc_patch_size != tese_patch_size) {
+                         "OutputVertices (patch size) is not specified in either of tessellation stages");
+    } else if (tesc_patch_size != spirv::kInvalidValue && tese_patch_size != spirv::kInvalidValue &&
+               tesc_patch_size != tese_patch_size) {
         const LogObjectList objlist(tesc_module_state.handle(), tese_module_state.handle());
         skip |= LogError("VUID-VkGraphicsPipelineCreateInfo-pStages-00735", objlist, create_info_loc,
-                         "Output patch size specified in tessellation control shader is %" PRIu32
-                         ", but subdivison type specified in tessellation evaluation shader is %" PRIu32,
+                         "OutputVertices (patch size) specified in tessellation control shader is %" PRIu32
+                         ", but OutputVertices specified in tessellation evaluation shader is %" PRIu32,
                          tesc_patch_size, tese_patch_size);
     }
     return skip;
@@ -730,7 +832,7 @@ bool CoreChecks::ValidateGraphicsPipelineShaderState(const vvl::Pipeline &pipeli
     // chained together in the pipeline. Note, we could be in the PreRaster GPL path, so there may not be a Fragment Shader see
     // https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8443
     if (pipeline.stage_states.size() > 1) {
-        const size_t not_found = vvl::kU32Max;
+        const size_t not_found = vvl::kNoIndex32;
         auto get_stage = [&pipeline, not_found = not_found](VkShaderStageFlagBits stage) {
             for (size_t i = 0; i < pipeline.stage_states.size(); i++) {
                 if (pipeline.stage_states[i].GetStage() == stage) {
