@@ -1,7 +1,7 @@
-/* Copyright (c) 2015-2025 The Khronos Group Inc.
- * Copyright (c) 2015-2025 Valve Corporation
- * Copyright (c) 2015-2025 LunarG, Inc.
- * Copyright (C) 2015-2025 Google Inc.
+/* Copyright (c) 2015-2026 The Khronos Group Inc.
+ * Copyright (c) 2015-2026 Valve Corporation
+ * Copyright (c) 2015-2026 LunarG, Inc.
+ * Copyright (C) 2015-2026 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include "stateless/stateless_validation.h"
 #include "generated/enum_flag_bits.h"
 #include "generated/dispatch_functions.h"
+#include "containers/container_utils.h"
 
 namespace stateless {
 // Traits objects to allow string_join to operate on collections of const char *
@@ -90,7 +91,7 @@ bool Instance::ValidateExtensionReqs(const ExtensionState &extensions, const cha
     // Check against the required list in the info
     std::vector<const char *> missing;
     for (const auto &req : info.requirements) {
-        if (!(extensions.*(req.enabled))) {
+        if (!IsExtEnabled(extensions.*(req.enabled))) {
             missing.push_back(req.name);
         }
     }
@@ -107,7 +108,7 @@ bool Instance::ValidateExtensionReqs(const ExtensionState &extensions, const cha
 ExtEnabled ExtensionStateByName(const DeviceExtensions &extensions, vvl::Extension extension) {
     auto info = extensions.GetInfo(extension);
     // unknown extensions can't be enabled in extension struct
-    ExtEnabled state = info.state ? extensions.*(info.state) : kNotEnabled;
+    ExtEnabled state = info.state ? extensions.*(info.state) : kNotSupported;
     return state;
 }
 
@@ -294,7 +295,7 @@ void Instance::CommonPostCallRecordEnumeratePhysicalDevice(const VkPhysicalDevic
             ext_props.resize(ext_count);
             DispatchEnumerateDeviceExtensionProperties(phys_device, nullptr, &ext_count, ext_props.data());
 
-            DeviceExtensions phys_dev_exts(extensions, phys_dev_props->apiVersion, ext_props);
+            DeviceExtensions phys_dev_exts(extensions, api_version, ext_props);
             physical_device_extensions[phys_device] = std::move(phys_dev_exts);
         }
     }
@@ -335,8 +336,6 @@ void Instance::PreCallRecordDestroyInstance(VkInstance instance, const VkAllocat
 }
 
 void Device::FinishDeviceSetup(const VkDeviceCreateInfo *pCreateInfo, const Location &loc) {
-    memcpy(&device_limits, &phys_dev_props.limits, sizeof(VkPhysicalDeviceLimits));
-
     std::vector<VkExtensionProperties> ext_props{};
     uint32_t ext_count = 0;
     DispatchEnumerateDeviceExtensionProperties(physical_device, nullptr, &ext_count, nullptr);
@@ -350,6 +349,8 @@ void Device::FinishDeviceSetup(const VkDeviceCreateInfo *pCreateInfo, const Loca
             scissor_exclusive_extension_version = prop.specVersion;
         }
     }
+
+    has_zero_queues = pCreateInfo->queueCreateInfoCount == 0;
 }
 
 bool Instance::manual_PreCallValidateCreateDevice(VkPhysicalDevice physicalDevice, const VkDeviceCreateInfo *pCreateInfo,
@@ -444,9 +445,9 @@ bool Instance::manual_PreCallValidateCreateDevice(VkPhysicalDevice physicalDevic
     const VkPhysicalDeviceFeatures *features = features2 ? &features2->features : pCreateInfo->pEnabledFeatures;
 
     if (const auto *robustness2_features =
-            vku::FindStructInPNextChain<VkPhysicalDeviceRobustness2FeaturesEXT>(pCreateInfo->pNext)) {
+            vku::FindStructInPNextChain<VkPhysicalDeviceRobustness2FeaturesKHR>(pCreateInfo->pNext)) {
         if (features && robustness2_features->robustBufferAccess2 && !features->robustBufferAccess) {
-            skip |= LogError("VUID-VkPhysicalDeviceRobustness2FeaturesEXT-robustBufferAccess2-04000", physicalDevice,
+            skip |= LogError("VUID-VkPhysicalDeviceRobustness2FeaturesKHR-robustBufferAccess2-04000", physicalDevice,
                              error_obj.location, "If robustBufferAccess2 is enabled then robustBufferAccess must be enabled.");
         }
     }
@@ -486,8 +487,10 @@ bool Instance::manual_PreCallValidateCreateDevice(VkPhysicalDevice physicalDevic
             if (IsValueIn(current->sType, illegal_feature_structs_with_11)) {
                 skip |= LogError("VUID-VkDeviceCreateInfo-pNext-02829", physicalDevice, error_obj.location,
                                  "If the pNext chain includes a VkPhysicalDeviceVulkan11Features structure, then "
-                                 "it must not include a %s structure.\n%s",
-                                 string_VkStructureType(current->sType),
+                                 "it must not include a %s structure. The features in %s were promoted in Vulkan 1.1 and is also "
+                                 "found in VkPhysicalDeviceVulkan11Features. To prevent one feature setting something to VK_TRUE "
+                                 "and the other to VK_FALSE, only one struct containing the feature is allowed.\n%s",
+                                 string_VkStructureName(current->sType), string_VkStructureName(current->sType),
                                  PrintPNextChain(Struct::VkDeviceCreateInfo, pCreateInfo->pNext).c_str());
                 break;
             }
@@ -524,8 +527,10 @@ bool Instance::manual_PreCallValidateCreateDevice(VkPhysicalDevice physicalDevic
             if (IsValueIn(current->sType, illegal_feature_structs_with_12)) {
                 skip |= LogError("VUID-VkDeviceCreateInfo-pNext-02830", physicalDevice, create_info_loc.dot(Field::pNext),
                                  "chain includes a VkPhysicalDeviceVulkan12Features structure, then it must not "
-                                 "include a %s structure\n%s",
-                                 string_VkStructureType(current->sType),
+                                 "include a %s structure. The features in %s were promoted in Vulkan 1.2 and is also found in "
+                                 "VkPhysicalDeviceVulkan12Features. To prevent one feature setting something to VK_TRUE and the "
+                                 "other to VK_FALSE, only one struct containing the feature is allowed.\n%s",
+                                 string_VkStructureName(current->sType), string_VkStructureName(current->sType),
                                  PrintPNextChain(Struct::VkDeviceCreateInfo, pCreateInfo->pNext).c_str());
                 break;
             }
@@ -601,8 +606,10 @@ bool Instance::manual_PreCallValidateCreateDevice(VkPhysicalDevice physicalDevic
             if (IsValueIn(current->sType, illegal_feature_structs_with_13)) {
                 skip |= LogError("VUID-VkDeviceCreateInfo-pNext-06532", physicalDevice, create_info_loc.dot(Field::pNext),
                                  "chain includes a VkPhysicalDeviceVulkan13Features structure, then it must not "
-                                 "include a %s structure\n%s",
-                                 string_VkStructureType(current->sType),
+                                 "include a %s structure. The features in %s were promoted in Vulkan 1.3 and is also found in "
+                                 "VkPhysicalDeviceVulkan13Features. To prevent one feature setting something to VK_TRUE and the "
+                                 "other to VK_FALSE, only one struct containing the feature is allowed.\n%s",
+                                 string_VkStructureName(current->sType), string_VkStructureName(current->sType),
                                  PrintPNextChain(Struct::VkDeviceCreateInfo, pCreateInfo->pNext).c_str());
                 break;
             }
@@ -632,13 +639,21 @@ bool Instance::manual_PreCallValidateCreateDevice(VkPhysicalDevice physicalDevic
             if (IsValueIn(current->sType, illegal_feature_structs_with_14)) {
                 skip |= LogError("VUID-VkDeviceCreateInfo-pNext-10360", physicalDevice, create_info_loc.dot(Field::pNext),
                                  "chain includes a VkPhysicalDeviceVulkan14Features structure, then it must not "
-                                 "include a %s structure\n%s",
-                                 string_VkStructureType(current->sType),
+                                 "include a %s structure. The features in %s were promoted in Vulkan 1.4 and is also found in "
+                                 "VkPhysicalDeviceVulkan14Features. To prevent one feature setting something to VK_TRUE and the "
+                                 "other to VK_FALSE, only one struct containing the feature is allowed.\n%s",
+                                 string_VkStructureName(current->sType), string_VkStructureName(current->sType),
                                  PrintPNextChain(Struct::VkDeviceCreateInfo, pCreateInfo->pNext).c_str());
                 break;
             }
             current = reinterpret_cast<const VkBaseOutStructure *>(current->pNext);
         }
+        if (vulkan_14_features->pushDescriptor == VK_FALSE &&
+            enabled_extensions.find(vvl::Extension::_VK_KHR_push_descriptor) != enabled_extensions.end()) {
+            skip |= LogError("VUID-VkDeviceCreateInfo-ppEnabledExtensionNames-10858", physicalDevice, error_obj.location,
+                             "%s is enabled but VkPhysicalDeviceVulkan14Features::pushDescriptor is not VK_TRUE.",
+                             VK_KHR_PUSH_DESCRIPTOR_EXTENSION_NAME);
+        }
     }
 
     // Validate pCreateInfo->pQueueCreateInfos
@@ -1032,4 +1047,43 @@ bool Instance::manual_PreCallValidateGetPhysicalDeviceProperties2(VkPhysicalDevi
     }
     return skip;
 }
+
+bool Instance::ValidateGetPhysicalDeviceFormatProperties2(VkPhysicalDevice physicalDevice, VkFormat format,
+                                                          VkFormatProperties2 *pFormatProperties, const Context &context) const {
+    bool skip = false;
+
+    if (IsValueIn(format, {VK_FORMAT_G8_B8R8_2PLANE_444_UNORM, VK_FORMAT_G10X6_B10X6R10X6_2PLANE_444_UNORM_3PACK16,
+                           VK_FORMAT_G12X4_B12X4R12X4_2PLANE_444_UNORM_3PACK16, VK_FORMAT_G16_B16R16_2PLANE_444_UNORM})) {
+        const auto &exposed_extensions = physical_device_extensions.at(physicalDevice);
+
+        if (api_version < VK_API_VERSION_1_3 && !exposed_extensions.vk_khr_maintenance5 &&
+            !exposed_extensions.vk_ext_ycbcr_2plane_444_formats) {
+            const char *vuid = context.error_obj.location.function == Func::vkGetPhysicalDeviceFormatProperties
+                                   ? "VUID-vkGetPhysicalDeviceFormatProperties-None-12272"
+                                   : "VUID-vkGetPhysicalDeviceFormatProperties2-None-12273";
+            skip |=
+                LogError(vuid, physicalDevice, context.error_obj.location.dot(Field::format), "is %s.", string_VkFormat(format));
+        }
+    }
+
+    return skip;
+}
+
+bool Instance::manual_PreCallValidateGetPhysicalDeviceFormatProperties(VkPhysicalDevice physicalDevice, VkFormat format,
+                                                                       VkFormatProperties *pFormatProperties,
+                                                                       const Context &context) const {
+    if (!pFormatProperties) {
+        return false;
+    }
+    VkFormatProperties2 format_props_2 = vku::InitStructHelper();
+    format_props_2.formatProperties = *pFormatProperties;
+    return ValidateGetPhysicalDeviceFormatProperties2(physicalDevice, format, &format_props_2, context);
+}
+
+bool Instance::manual_PreCallValidateGetPhysicalDeviceFormatProperties2(VkPhysicalDevice physicalDevice, VkFormat format,
+                                                                        VkFormatProperties2 *pFormatProperties,
+                                                                        const Context &context) const {
+    return ValidateGetPhysicalDeviceFormatProperties2(physicalDevice, format, pFormatProperties, context);
+}
+
 }  // namespace stateless
