@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2019-2025 Valve Corporation
- * Copyright (c) 2019-2025 LunarG, Inc.
+ * Copyright (c) 2019-2026 Valve Corporation
+ * Copyright (c) 2019-2026 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,8 @@
 #include "sync/sync_image.h"
 #include "sync/sync_reporting.h"
 
+namespace syncval {
+
 AcquiredImage::AcquiredImage(const PresentedImage& presented, ResourceUsageTag acq_tag)
     : image(presented.image), generator(presented.range_gen), present_tag(presented.tag), acquire_tag(acq_tag) {}
 
@@ -159,7 +161,7 @@ std::optional<SignalInfo> SignalsUpdate::OnTimelineWait(VkSemaphore semaphore, u
     return resolving_signal;  // empty result if it is a wait-before-signal
 }
 
-void syncval_state::SwapchainSubState::RecordPresentedImage(PresentedImage&& presented_image) {
+void SwapchainSubState::RecordPresentedImage(PresentedImage&& presented_image) {
     // All presented images are stored within the swapchain until the are reaquired.
     const uint32_t image_index = presented_image.image_index;
     if (image_index >= presented.size()) presented.resize(image_index + 1);
@@ -169,7 +171,7 @@ void syncval_state::SwapchainSubState::RecordPresentedImage(PresentedImage&& pre
 }
 
 // We move from the presented images array 1) so we don't copy shared_ptr, and 2) to mark it acquired
-PresentedImage syncval_state::SwapchainSubState::MovePresentedImage(uint32_t image_index) {
+PresentedImage SwapchainSubState::MovePresentedImage(uint32_t image_index) {
     if (presented.size() <= image_index) presented.resize(image_index + 1);
     PresentedImage ret_val = std::move(presented[image_index]);
     if (ret_val.Invalid()) {
@@ -180,7 +182,7 @@ PresentedImage syncval_state::SwapchainSubState::MovePresentedImage(uint32_t ima
     return ret_val;
 }
 
-void syncval_state::SwapchainSubState::GetPresentBatches(std::vector<QueueBatchContext::Ptr>& batches) const {
+void SwapchainSubState::GetPresentBatches(std::vector<QueueBatchContext::Ptr>& batches) const {
     for (const auto& presented_image : presented) {
         if (presented_image.batch) {
             batches.push_back(presented_image.batch);
@@ -191,7 +193,7 @@ void syncval_state::SwapchainSubState::GetPresentBatches(std::vector<QueueBatchC
 class ApplySemaphoreBarrierAction {
   public:
     ApplySemaphoreBarrierAction(const SemaphoreScope& signal, const SemaphoreScope& wait) : signal_(signal), wait_(wait) {}
-    void operator()(ResourceAccessState* access) const { access->ApplySemaphore(signal_, wait_); }
+    void operator()(AccessState* access) const { access->ApplySemaphore(signal_, wait_); }
 
   private:
     const SemaphoreScope& signal_;
@@ -201,55 +203,86 @@ class ApplySemaphoreBarrierAction {
 class ApplyAcquireNextSemaphoreAction {
   public:
     ApplyAcquireNextSemaphoreAction(const SyncExecScope& wait_scope, ResourceUsageTag acquire_tag)
-        : barrier_(1, SyncBarrier(getPresentSrcScope(), getPresentValidAccesses(), wait_scope, SyncAccessFlags())),
-          acq_tag_(acquire_tag) {}
-    void operator()(ResourceAccessState* access) const {
+        : barrier_(GetAcquireBarrier(wait_scope)), acq_tag_(acquire_tag) {}
+    void operator()(AccessState* access) const {
         // Note that the present operations may or may not be present, given that the fence wait may have cleared them out.
         // Also, if a subsequent present has happened, we *don't* want to protect that...
         if (access->LastWriteTag() <= acq_tag_) {
-            access->ApplyBarriersImmediate(barrier_);
+            access->ApplyBarrier(BarrierScope(barrier_), barrier_);
         }
     }
 
   private:
+    static SyncBarrier GetAcquireBarrier(const SyncExecScope& wait_scope) {
+        SyncBarrier barrier;
+        barrier.src_exec_scope = getPresentSrcScope();
+        barrier.src_access_scope = getPresentValidAccesses();
+        barrier.dst_exec_scope = wait_scope;
+        return barrier;
+    }
     // kPresentSrcScope/kPresentValidAccesses cannot be regular global variables, because they use global
     // variables from another compilation unit (through syncStageAccessMaskByStageBit() call) for initialization,
     // and initialization of globals between compilation units is undefined. Instead they get initialized
     // on the first use (it's important to ensure this first use is also not initialization of some global!).
-    const SyncExecScope& getPresentSrcScope() const {
-        static const SyncExecScope kPresentSrcScope =
-            SyncExecScope(VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL,  // mask_param (unused)
-                          VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL,  // exec_scope
-                          getPresentValidAccesses());                      // valid_accesses
+    static const SyncExecScope& getPresentSrcScope() {
+        static const SyncExecScope kPresentSrcScope{VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL,  // mask_param (unused)
+                                                    VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL,  // exec_scope
+                                                    getPresentValidAccesses()};                      // valid_accesses
         return kPresentSrcScope;
     }
-    const SyncAccessFlags& getPresentValidAccesses() const {
-        static const SyncAccessFlags kPresentValidAccesses =
-            SyncAccessFlags(SyncStageAccess::AccessScopeByStage(VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL));
+    static const SyncAccessFlags& getPresentValidAccesses() {
+        static const SyncAccessFlags kPresentValidAccesses = SYNC_PRESENT_ENGINE_BIT_SYNCVAL_PRESENT_ACQUIRE_READ_BIT_SYNCVAL |
+                                                             SYNC_PRESENT_ENGINE_BIT_SYNCVAL_PRESENT_PRESENTED_BIT_SYNCVAL;
         return kPresentValidAccesses;
     }
 
   private:
-    std::vector<SyncBarrier> barrier_;
+    SyncBarrier barrier_;
     ResourceUsageTag acq_tag_;
 };
 
+void LastSynchronizedPresent::Update(VkSwapchainKHR swapchain, ResourceUsageTag present_tag) {
+    for (auto& entry : per_swapchain) {
+        if (entry.first == swapchain) {
+            entry.second = std::max(entry.second, present_tag);
+            return;
+        }
+    }
+    per_swapchain.emplace_back(std::make_pair(swapchain, present_tag));
+}
+
+void LastSynchronizedPresent::Merge(const LastSynchronizedPresent& other) {
+    for (const auto& other_entry : other.per_swapchain) {
+        Update(other_entry.first, other_entry.second);
+    }
+}
+
+void LastSynchronizedPresent::OnDestroySwapchain(VkSwapchainKHR swapchain) {
+    // TODO: add erase() to small_vector and use intead in the following code
+    const auto copy = per_swapchain;
+    per_swapchain.resize(0);
+    for (const auto& entry : copy) {
+        if (entry.first != swapchain) {
+            per_swapchain.emplace_back(entry);
+        }
+    }
+}
+
 QueueBatchContext::QueueBatchContext(const SyncValidator& sync_state, const QueueSyncState& queue_state)
     : CommandExecutionContext(sync_state, queue_state.GetQueueFlags()),
       queue_state_(&queue_state),
       tag_range_(0, 0),
+      access_context_(sync_state),
       current_access_context_(&access_context_),
-      batch_log_(),
       queue_sync_tag_(sync_state.GetQueueIdLimit(), ResourceUsageTag(0)) {
     sync_state_.stats.AddQueueBatchContext();
 }
 
 QueueBatchContext::QueueBatchContext(const SyncValidator& sync_state)
     : CommandExecutionContext(sync_state, 0),
-      queue_state_(),
       tag_range_(0, 0),
+      access_context_(sync_state),
       current_access_context_(&access_context_),
-      batch_log_(),
       queue_sync_tag_(sync_state.GetQueueIdLimit(), ResourceUsageTag(0)) {
     sync_state_.stats.AddQueueBatchContext();
 }
@@ -277,42 +310,107 @@ void QueueBatchContext::ResolveSubmittedCommandBuffer(const AccessContext& recor
 VulkanTypedHandle QueueBatchContext::Handle() const { return queue_state_->Handle(); }
 
 template <typename Predicate>
-void QueueBatchContext::ApplyPredicatedWait(Predicate& predicate) {
-    access_context_.EraseIf([&predicate](ResourceAccessRangeMap::value_type& access) {
-        // Apply..Wait returns true if the waited access is empty...
-        return access.second.ApplyPredicatedWait<Predicate>(predicate);
+void QueueBatchContext::ApplyPredicatedWait(Predicate& predicate, const LastSynchronizedPresent& last_synchronized_present) {
+    access_context_.EraseIf([this, &last_synchronized_present, &predicate](AccessMap::value_type& access) {
+        AccessState& access_state = access.second;
+
+        // Tell EraseIf to remove present accesses that are already synchronized according to LastSynchronizedPresent
+        if (access_state.HasWriteOp() && access_state.LastWrite().IsPresent()) {
+            const ResourceUsageRecord& usage_record = *batch_log_.GetAccessRecord(access_state.LastWriteTag()).record;
+            assert(usage_record.alt_usage);
+            assert(usage_record.alt_usage.GetCommand() == vvl::Func::vkQueuePresentKHR);
+            const VkSwapchainKHR swapchain = usage_record.alt_usage.GetSwapchainHandle();
+            for (const auto& [synchronized_swapchain, synchronized_present_tag] : last_synchronized_present.per_swapchain) {
+                // NOTE: it is important to check that access belongs to a specific swapchain and not only
+                // compare the tag values. It is possible to have accesses from a different swapchain that
+                // are *not* synchronized and have tag values that satisfy tag comparison check.
+                if (swapchain == synchronized_swapchain && access_state.LastWriteTag() <= synchronized_present_tag) {
+                    return true;
+                }
+            }
+        }
+
+        // Tell eraseIf to remove accesses that become empty after applying predicate
+        return access_state.ClearPredicatedAccesses<Predicate>(predicate);
     });
 }
 
-void QueueBatchContext::ApplyTaggedWait(QueueId queue_id, ResourceUsageTag tag) {
-    const bool any_queue = (queue_id == kQueueAny);
+struct WaitQueueTagPredicate {
+    QueueId queue;
+    ResourceUsageTag tag;
 
-    if (any_queue) {
-        // This isn't just avoid an unneeded test, but to allow *all* queues to to be waited in a single pass
-        // (and it does avoid doing the same test for every access, as well as avoiding the need for the predicate
-        // to grok Queue/Device/Wait differences.
-        ResourceAccessState::WaitTagPredicate predicate{tag};
-        ApplyPredicatedWait(predicate);
-    } else {
-        ResourceAccessState::WaitQueueTagPredicate predicate{queue_id, tag};
-        ApplyPredicatedWait(predicate);
+    bool operator()(const ReadState& read_access) const {
+        return read_access.queue == queue && read_access.tag <= tag &&
+               read_access.stage != VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL;
     }
+    bool operator()(const AccessState& access) const {
+        if (!access.HasWriteOp()) {
+            return false;
+        }
+        const WriteState& write_state = access.LastWrite();
+        return write_state.queue == queue && write_state.tag <= tag &&
+               write_state.access_index != SYNC_PRESENT_ENGINE_SYNCVAL_PRESENT_PRESENTED_SYNCVAL;
+    }
+};
 
-    // SwapChain acquire QBC's have no queue, but also, events are always empty.
-    if (queue_state_ && (queue_id == GetQueueId() || any_queue)) {
+struct DeviceWaitPredicate {
+    bool operator()(const ReadState& read_access) const {
+        return read_access.stage != VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL;
+    }
+    bool operator()(const AccessState& access) const {
+        if (!access.HasWriteOp()) {
+            return false;
+        }
+        return access.LastWrite().access_index != SYNC_PRESENT_ENGINE_SYNCVAL_PRESENT_PRESENTED_SYNCVAL;
+    }
+};
+
+// Present operations matching only the *exactly* tagged present and acquire operations
+struct WaitAcquirePredicate {
+    ResourceUsageTag present_tag;
+    ResourceUsageTag acquire_tag;
+
+    bool operator()(const ReadState& read_access) const {
+        return read_access.tag == acquire_tag && read_access.stage == VK_PIPELINE_STAGE_2_PRESENT_ENGINE_BIT_SYNCVAL;
+    }
+    bool operator()(const AccessState& access) const {
+        if (!access.HasWriteOp()) {
+            return false;
+        }
+        const WriteState& write_state = access.LastWrite();
+        return write_state.tag == present_tag && write_state.access_index == SYNC_PRESENT_ENGINE_SYNCVAL_PRESENT_PRESENTED_SYNCVAL;
+    }
+};
+
+void QueueBatchContext::ApplyTaggedWait(QueueId queue_id, ResourceUsageTag tag,
+                                        const LastSynchronizedPresent& last_synchronized_present) {
+    WaitQueueTagPredicate predicate{queue_id, tag};
+    ApplyPredicatedWait(predicate, last_synchronized_present);
+
+    // SwapChain acquire batch contexts have no queue
+    if (queue_state_ && queue_id == GetQueueId()) {
         events_context_.ApplyTaggedWait(queue_state_->GetQueueFlags(), tag);
     }
 }
 
-void QueueBatchContext::ApplyAcquireWait(const AcquiredImage& acquired) {
-    ResourceAccessState::WaitAcquirePredicate predicate{acquired.present_tag, acquired.acquire_tag};
-    ApplyPredicatedWait(predicate);
+void QueueBatchContext::ApplyDeviceWait(const LastSynchronizedPresent& last_synchronized_present) {
+    DeviceWaitPredicate predicate;
+    ApplyPredicatedWait(predicate, last_synchronized_present);
+
+    // SwapChain acquire batch contexts have no queue
+    if (queue_state_) {
+        events_context_.ApplyTaggedWait(queue_state_->GetQueueFlags(), ResourceUsageRecord::kMaxIndex);
+    }
 }
 
-void QueueBatchContext::OnResourceDestroyed(const ResourceAccessRange& resource_range) {
+void QueueBatchContext::ApplyAcquireWait(const AcquiredImage& acquired) {
+    WaitAcquirePredicate predicate{acquired.present_tag, acquired.acquire_tag};
+    ApplyPredicatedWait(predicate, {});
+}
+
+void QueueBatchContext::OnResourceDestroyed(const AccessRange& resource_range) {
     // Remove all accesses associated with the resource being destroyed
-    access_context_.EraseIf(
-        [&resource_range](ResourceAccessRangeMap::value_type& access) { return resource_range.includes(access.first); });
+    access_context_.EraseIf([&resource_range](AccessMap::value_type& access) { return resource_range.includes(access.first); });
 }
 
 void QueueBatchContext::BeginRenderPassReplaySetup(ReplayState& replay, const SyncOpBeginRenderPass& begin_op) {
@@ -352,15 +450,13 @@ void QueueBatchContext::ResolvePresentSemaphoreWait(const SignalInfo& signal_inf
 
     // For each presented image
     for (const auto& presented : presented_images) {
-        // Need a copy that can be used as the pseudo-iterator...
-        subresource_adapter::ImageRangeGenerator range_gen(presented.range_gen);
         if (signal_scope.queue == wait_scope.queue) {
             // If signal queue == wait queue, signal is treated as a memory barrier with an access scope equal to the
             // valid accesses for the sync scope.
-            access_context_.ResolveFromContext(sem_same_queue_op, from_context, range_gen);
+            access_context_.ResolveFromContext(sem_same_queue_op, from_context, presented.range_gen);
             access_context_.ResolveFromContext(noop_barrier_op, from_context);
         } else {
-            access_context_.ResolveFromContext(sem_not_same_queue_op, from_context, range_gen);
+            access_context_.ResolveFromContext(sem_not_same_queue_op, from_context, presented.range_gen);
             access_context_.ResolveFromContext(noop_sem_op, from_context);
         }
     }
@@ -375,9 +471,17 @@ void QueueBatchContext::ResolveSubmitSemaphoreWait(const SignalInfo& signal_info
 
     const AccessContext& from_context = signal_info.batch->access_context_;
     if (signal_info.acquired_image) {
+        const AcquiredImage& acquired_image = *signal_info.acquired_image;
+
+        // Update last synchronized presentation
+        if (acquired_image.present_tag != kInvalidTag) {
+            const VkSwapchainKHR swapchain = acquired_image.image->create_from_swapchain;
+            last_synchronized_present.Update(swapchain, acquired_image.present_tag);
+        }
+
         // Import the *presenting* batch, but replacing presenting with acquired.
-        ApplyAcquireNextSemaphoreAction apply_acq(wait_scope, signal_info.acquired_image->acquire_tag);
-        access_context_.ResolveFromContext(apply_acq, from_context, signal_info.acquired_image->generator);
+        ApplyAcquireNextSemaphoreAction apply_acq(wait_scope, acquired_image.acquire_tag);
+        access_context_.ResolveFromContext(apply_acq, from_context, acquired_image.generator);
 
         // Grab the reset of the presenting QBC, with no effective barrier, won't overwrite the acquire, as the tag is newer
         SyncBarrier noop_barrier;
@@ -405,6 +509,8 @@ void QueueBatchContext::ResolveLastBatch(const QueueBatchContext::ConstPtr& last
     // If there are no semaphores to the previous batch, make sure a "submit order" non-barriered import is done
     access_context_.ResolveFromContext(last_batch->access_context_);
     ImportTags(*last_batch);
+
+    last_synchronized_present.Merge(last_batch->last_synchronized_present);
 }
 
 void QueueBatchContext::ImportTags(const QueueBatchContext& from) {
@@ -446,7 +552,7 @@ bool QueueBatchContext::DoQueuePresentValidate(const Location& loc, const Presen
 
             LogObjectList objlist(queue_state_->Handle(), swapchain_handle, image_handle);
 
-            std::stringstream ss;
+            std::ostringstream ss;
             ss << "swapchain image " << presented.image_index << " (";
             ss << sync_state_.FormatHandle(image_handle);
             ss << " from " << sync_state_.FormatHandle(swapchain_handle) << ")";
@@ -467,7 +573,8 @@ void QueueBatchContext::DoPresentOperations(const PresentedImages& presented_ima
     // For present, tagging is internal to the presented image record.
     for (const auto& presented : presented_images) {
         // Update memory state
-        presented.UpdateMemoryAccess(SYNC_PRESENT_ENGINE_SYNCVAL_PRESENT_PRESENTED_SYNCVAL, presented.tag, access_context_);
+        presented.UpdateMemoryAccess(SYNC_PRESENT_ENGINE_SYNCVAL_PRESENT_PRESENTED_SYNCVAL, presented.tag, access_context_,
+                                     SyncFlag::kPresent);
     }
 }
 
@@ -619,12 +726,12 @@ bool QueueBatchContext::ValidateSubmit(const std::vector<CommandBufferConstPtr>&
     uint32_t tag_count = 0;
     for (const auto& cb : command_buffers) {
         if (!cb) continue;
-        tag_count += static_cast<uint32_t>(syncval_state::SubState(*cb).access_context.GetTagCount());
+        tag_count += static_cast<uint32_t>(SubState(*cb).access_context.GetTagCount());
     }
     batch.base_tag = SetupBatchTags(tag_count);
 
     for (size_t index = 0; index < command_buffers.size(); index++) {
-        const auto& cb = syncval_state::SubState(*command_buffers[index]);
+        const auto& cb = SubState(*command_buffers[index]);
         // Validate and resolve command buffers that has tagged commands
         const CommandBufferAccessContext& access_context = cb.access_context;
         if (access_context.GetTagCount() > 0) {
@@ -645,10 +752,18 @@ QueueBatchContext::PresentResourceRecord::Base_::Record QueueBatchContext::Prese
     return std::make_unique<PresentResourceRecord>(presented_);
 }
 
+VkSwapchainKHR QueueBatchContext::PresentResourceRecord::GetSwapchainHandle() const {
+    return presented_.image->create_from_swapchain;
+}
+
 QueueBatchContext::AcquireResourceRecord::Base_::Record QueueBatchContext::AcquireResourceRecord::MakeRecord() const {
     return std::make_unique<AcquireResourceRecord>(presented_, acquire_tag_, command_);
 }
 
+VkSwapchainKHR QueueBatchContext::AcquireResourceRecord::GetSwapchainHandle() const {
+    return presented_.image->create_from_swapchain;
+}
+
 std::vector<QueueBatchContext::ConstPtr> SyncValidator::GetLastBatches(
     std::function<bool(const QueueBatchContext::ConstPtr&)> filter) const {
     std::vector<QueueBatchContext::ConstPtr> snapshot;
@@ -696,6 +811,11 @@ void SyncValidator::ClearPending() const {
 // atomic... but as the ops are per submit, the performance cost is negible for the peace of mind.
 uint64_t QueueSyncState::ReserveSubmitId() const { return submit_index_.fetch_add(1); }
 
+const LastSynchronizedPresent& QueueSyncState::GetLastSynchronizedPresent() const {
+    static const LastSynchronizedPresent empty;
+    return last_batch_ ? last_batch_->last_synchronized_present : empty;
+}
+
 void QueueSyncState::SetPendingLastBatch(QueueBatchContext::Ptr&& last) const { pending_last_batch_ = std::move(last); }
 
 // Since we're updating the QueueSync state, this is Record phase and the access log needs to point to the global one
@@ -830,7 +950,7 @@ BatchAccessLog::AccessRecord BatchAccessLog::CBSubmitLog::GetAccessRecord(Resour
     assert(log_);
     assert(index < log_->size());
     const ResourceUsageRecord* record = &(*log_)[index];
-    const auto debug_name_provider = (record->label_command_index == vvl::kU32Max) ? nullptr : this;
+    const auto debug_name_provider = (record->label_command_index == vvl::kNoIndex32) ? nullptr : this;
     return AccessRecord{&batch_, record, debug_name_provider};
 }
 
@@ -863,7 +983,7 @@ void PresentedImage::ExportToSwapchain(SyncValidator&) {  // Include this argume
     // If the swapchain is dead just ignore the present
     auto swap_lock = swapchain_state.lock();
     if (vvl::StateObject::Invalid(swap_lock)) return;
-    auto& sub_state = syncval_state::SubState(*swap_lock);
+    auto& sub_state = SubState(*swap_lock);
     sub_state.RecordPresentedImage(std::move(*this));
 }
 
@@ -878,12 +998,15 @@ void PresentedImage::SetImage(uint32_t at_index) {
         range_gen = ImageRangeGen();
     } else {
         // For valid images create the type/range_gen to used to scope the semaphore operations
-        const auto& sub_state = syncval_state::SubState(*image);
+        const auto& sub_state = SubState(*image);
         range_gen = sub_state.MakeImageRangeGen(image->full_range, false);
     }
 }
 
-void PresentedImage::UpdateMemoryAccess(SyncAccessIndex usage, ResourceUsageTag tag, AccessContext& access_context) const {
-    // Intentional copy. The range_gen argument is not copied by the Update... call below
-    access_context.UpdateAccessState(range_gen, usage, SyncOrdering::kNonAttachment, ResourceUsageTagEx{tag});
+void PresentedImage::UpdateMemoryAccess(SyncAccessIndex usage, ResourceUsageTag tag, AccessContext& access_context,
+                                        SyncFlags flags) const {
+    ImageRangeGen mutable_range_gen = range_gen;
+    access_context.UpdateAccessState(mutable_range_gen, usage, SyncOrdering::kNonAttachment, ResourceUsageTagEx{tag}, flags);
 }
+
+}  // namespace syncval
