@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2023-2024 The Khronos Group Inc.
+# Copyright (c) 2023-2025 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,260 +21,484 @@ from base_generator import BaseGenerator
 # This is a representation of all the dynamic state information
 dynamic_state_map = {
     "VK_DYNAMIC_STATE_VIEWPORT" : {
-        "command" : ["vkCmdSetViewport"]
+        "command" : ["vkCmdSetViewport"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_SCISSOR" : {
-        "command" : ["vkCmdSetScissor"]
+        "command" : ["vkCmdSetScissor"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_LINE_WIDTH" : {
-        "command" : ["vkCmdSetLineWidth"]
+        "command" : ["vkCmdSetLineWidth"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_BIAS" : {
         "command" : ["vkCmdSetDepthBias", "vkCmdSetDepthBias2EXT"],
-        "dependency" : ["rasterizerDiscardEnable", "depthBiasEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "depthBiasEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_BLEND_CONSTANTS" : {
-        "command" : ["vkCmdSetBlendConstants"]
+        "command" : ["vkCmdSetBlendConstants"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_BOUNDS" : {
         "command" : ["vkCmdSetDepthBounds"],
-        "dependency" : ["rasterizerDiscardEnable", "depthBoundsTestEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "depthBoundsTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_STENCIL_COMPARE_MASK" : {
         "command" : ["vkCmdSetStencilCompareMask"],
-        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_STENCIL_WRITE_MASK" : {
         "command" : ["vkCmdSetStencilWriteMask"],
-        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_STENCIL_REFERENCE" : {
         "command" : ["vkCmdSetStencilReference"],
-        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_CULL_MODE" : {
         "command" : ["vkCmdSetCullMode"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_FRONT_FACE" : {
         "command" : ["vkCmdSetFrontFace"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_PRIMITIVE_TOPOLOGY" : {
-        "command" : ["vkCmdSetPrimitiveTopology"]
+        "command" : ["vkCmdSetPrimitiveTopology"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["VertexInput"],
+        "pipeline" : "VkPipelineInputAssemblyStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT" : {
-        "command" : ["vkCmdSetViewportWithCount"]
+        "command" : ["vkCmdSetViewportWithCount"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT" : {
-        "command" : ["vkCmdSetScissorWithCount"]
+        "command" : ["vkCmdSetScissorWithCount"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_VERTEX_INPUT_BINDING_STRIDE" : {
-        "command" : ["vkCmdBindVertexBuffers2"]
+        "command" : ["vkCmdBindVertexBuffers2"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["VertexInput"],
+        "pipeline" : "VkPipelineVertexInputStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE" : {
         "command" : ["vkCmdSetDepthTestEnable"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE" : {
         "command" : ["vkCmdSetDepthWriteEnable"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "depthTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_COMPARE_OP" : {
         "command" : ["vkCmdSetDepthCompareOp"],
-        "dependency" : ["rasterizerDiscardEnable", "depthTestEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "depthTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_BOUNDS_TEST_ENABLE" : {
         "command" : ["vkCmdSetDepthBoundsTestEnable"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE" : {
         "command" : ["vkCmdSetStencilTestEnable"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_STENCIL_OP" : {
         "command" : ["vkCmdSetStencilOp"],
-        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "stencilTestEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineDepthStencilStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE" : {
-        "command" : ["vkCmdSetRasterizerDiscardEnable"]
+        "command" : ["vkCmdSetRasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE" : {
         "command" : ["vkCmdSetDepthBiasEnable"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE" : {
-        "command" : ["vkCmdSetPrimitiveRestartEnable"]
+        "command" : ["vkCmdSetPrimitiveRestartEnable"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["VertexInput"],
+        "pipeline" : "VkPipelineInputAssemblyStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_NV" : {
         "command" : ["vkCmdSetViewportWScalingNV"],
-        "dependency" : ["viewportWScalingEnable"]
+        "dependency" : ["viewportWScalingEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportWScalingStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_DISCARD_RECTANGLE_EXT" : {
         "command" : ["vkCmdSetDiscardRectangleEXT"],
-        "dependency" : ["rasterizerDiscardEnable", "discardRectangleEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "discardRectangleEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineDiscardRectangleStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT" : {
         "command" : ["vkCmdSetDiscardRectangleEnableEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineDiscardRectangleStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_DISCARD_RECTANGLE_MODE_EXT" : {
         "command" : ["vkCmdSetDiscardRectangleModeEXT"],
-        "dependency" : ["rasterizerDiscardEnable", "discardRectangleEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "discardRectangleEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineDiscardRectangleStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_EXT" : {
         "command" : ["vkCmdSetSampleLocationsEXT"],
-        "dependency" : ["rasterizerDiscardEnable", "sampleLocationsEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "sampleLocationsEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineSampleLocationsStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_VIEWPORT_SHADING_RATE_PALETTE_NV" : {
         "command" : ["vkCmdSetViewportShadingRatePaletteNV"],
-        "dependency" : ["rasterizerDiscardEnable", "shadingRateImageEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "shadingRateImageEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportShadingRateImageStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_VIEWPORT_COARSE_SAMPLE_ORDER_NV" : {
         "command" : ["vkCmdSetCoarseSampleOrderNV"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportCoarseSampleOrderStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_ENABLE_NV" : {
-        "command" : ["vkCmdSetExclusiveScissorEnableNV"]
+        "command" : ["vkCmdSetExclusiveScissorEnableNV"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportExclusiveScissorStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV" : {
         "command" : ["vkCmdSetExclusiveScissorNV"],
-        "dependency" : ["pExclusiveScissorEnables"]
+        "dependency" : ["pExclusiveScissorEnables"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportExclusiveScissorStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_FRAGMENT_SHADING_RATE_KHR" : {
         "command" : ["vkCmdSetFragmentShadingRateKHR"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization", "FragmentShader"],
+        "pipeline" : "VkPipelineFragmentShadingRateStateCreateInfoKHR",
     },
     "VK_DYNAMIC_STATE_LINE_STIPPLE" : {
         "command" : ["vkCmdSetLineStipple"],
-        "dependency" : ["rasterizerDiscardEnable", "stippledLineEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "stippledLineEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationLineStateCreateInfoKHR",
     },
     "VK_DYNAMIC_STATE_VERTEX_INPUT_EXT" : {
-        "command" : ["vkCmdSetVertexInputEXT"]
+        "command" : ["vkCmdSetVertexInputEXT"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["VertexInput"],
+        "pipeline" : "VkPipelineVertexInputStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_PATCH_CONTROL_POINTS_EXT" : {
-        "command" : ["vkCmdSetPatchControlPointsEXT"]
+        "command" : ["vkCmdSetPatchControlPointsEXT"],
+        "stage" : "VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineTessellationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_LOGIC_OP_EXT" : {
         "command" : ["vkCmdSetLogicOpEXT"],
-        "dependency" : ["rasterizerDiscardEnable", "logicOpEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "logicOpEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_COLOR_WRITE_ENABLE_EXT" : {
-        "command" : ["vkCmdSetColorWriteEnableEXT"]
+        "command" : ["vkCmdSetColorWriteEnableEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorWriteCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_TESSELLATION_DOMAIN_ORIGIN_EXT" : {
-        "command" : ["vkCmdSetTessellationDomainOriginEXT"]
+        "command" : ["vkCmdSetTessellationDomainOriginEXT"],
+        "stage" : "VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineTessellationDomainOriginStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT" : {
-        "command" : ["vkCmdSetDepthClampEnableEXT"]
+        "command" : ["vkCmdSetDepthClampEnableEXT"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_POLYGON_MODE_EXT" : {
-        "command" : ["vkCmdSetPolygonModeEXT"]
+        "command" : ["vkCmdSetPolygonModeEXT"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT" : {
-        "command" : ["vkCmdSetRasterizationSamplesEXT"]
+        "command" : ["vkCmdSetRasterizationSamplesEXT"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineMultisampleStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_SAMPLE_MASK_EXT" : {
-        "command" : ["vkCmdSetSampleMaskEXT"]
+        "command" : ["vkCmdSetSampleMaskEXT"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineMultisampleStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_ALPHA_TO_COVERAGE_ENABLE_EXT" : {
         "command" : ["vkCmdSetAlphaToCoverageEnableEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineMultisampleStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_ALPHA_TO_ONE_ENABLE_EXT" : {
         "command" : ["vkCmdSetAlphaToOneEnableEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineMultisampleStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_LOGIC_OP_ENABLE_EXT" : {
         "command" : ["vkCmdSetLogicOpEnableEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendStateCreateInfo",
     },
     "VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT" : {
-        "command" : ["vkCmdSetColorBlendEnableEXT"]
+        "command" : ["vkCmdSetColorBlendEnableEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendAttachmentState",
     },
     "VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT" : {
-        "command" : ["vkCmdSetColorBlendEquationEXT"]
+        "command" : ["vkCmdSetColorBlendEquationEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendAttachmentState",
     },
     "VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT" : {
-        "command" : ["vkCmdSetColorWriteMaskEXT"]
+        "command" : ["vkCmdSetColorWriteMaskEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendAttachmentState",
     },
     "VK_DYNAMIC_STATE_RASTERIZATION_STREAM_EXT" : {
-        "command" : ["vkCmdSetRasterizationStreamEXT"]
+        "command" : ["vkCmdSetRasterizationStreamEXT"],
+        "stage" : "VK_SHADER_STAGE_GEOMETRY_BIT",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationStateStreamCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_CONSERVATIVE_RASTERIZATION_MODE_EXT" : {
         "command" : ["vkCmdSetConservativeRasterizationModeEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationConservativeStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_EXTRA_PRIMITIVE_OVERESTIMATION_SIZE_EXT" : {
         "command" : ["vkCmdSetExtraPrimitiveOverestimationSizeEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationConservativeStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_DEPTH_CLIP_ENABLE_EXT" : {
-        "command" : ["vkCmdSetDepthClipEnableEXT"]
+        "command" : ["vkCmdSetDepthClipEnableEXT"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationDepthClipStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_ENABLE_EXT" : {
         "command" : ["vkCmdSetSampleLocationsEnableEXT"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineSampleLocationsStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT" : {
         "command" : ["vkCmdSetColorBlendAdvancedEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentOutput"],
+        "pipeline" : "VkPipelineColorBlendAdvancedStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_PROVOKING_VERTEX_MODE_EXT" : {
         "command" : ["vkCmdSetProvokingVertexModeEXT"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationProvokingVertexStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT" : {
         "command" : ["vkCmdSetLineRasterizationModeEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationLineStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT" : {
         "command" : ["vkCmdSetLineStippleEnableEXT"],
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_VERTEX_BIT",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineRasterizationLineStateCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_DEPTH_CLIP_NEGATIVE_ONE_TO_ONE_EXT" : {
         "command" : ["vkCmdSetDepthClipNegativeOneToOneEXT"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportDepthClipControlCreateInfoEXT",
     },
     "VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE_NV" : {
         "command" : ["vkCmdSetViewportWScalingEnableNV"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportWScalingStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_VIEWPORT_SWIZZLE_NV" : {
         "command" : ["vkCmdSetViewportSwizzleNV"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportSwizzleStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_ENABLE_NV" : {
         "command" : ["vkCmdSetCoverageToColorEnableNV"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineCoverageToColorStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_LOCATION_NV" : {
         "command" : ["vkCmdSetCoverageToColorLocationNV"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineCoverageToColorStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV" : {
         "command" : ["vkCmdSetCoverageModulationModeNV"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineCoverageModulationStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_ENABLE_NV" : {
         "command" : ["vkCmdSetCoverageModulationTableEnableNV"],
-        "dependency" : ["rasterizerDiscardEnable"]
+        "dependency" : ["rasterizerDiscardEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineCoverageModulationStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_NV" : {
         "command" : ["vkCmdSetCoverageModulationTableNV"],
-        "dependency" : ["rasterizerDiscardEnable", "coverageModulationTableEnable"]
+        "dependency" : ["rasterizerDiscardEnable", "coverageModulationTableEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineCoverageModulationStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV" : {
         "command" : ["vkCmdSetShadingRateImageEnableNV"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportShadingRateImageStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_REPRESENTATIVE_FRAGMENT_TEST_ENABLE_NV" : {
         "command" : ["vkCmdSetRepresentativeFragmentTestEnableNV"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader"],
+        "pipeline" : "VkPipelineRepresentativeFragmentTestStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_COVERAGE_REDUCTION_MODE_NV" : {
         "command" : ["vkCmdSetCoverageReductionModeNV"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["FragmentShader", "FragmentOutput"],
+        "pipeline" : "VkPipelineCoverageReductionStateCreateInfoNV",
     },
     "VK_DYNAMIC_STATE_ATTACHMENT_FEEDBACK_LOOP_ENABLE_EXT" : {
         "command" : ["vkCmdSetAttachmentFeedbackLoopEnableEXT"],
+        "stage" : "VK_SHADER_STAGE_FRAGMENT_BIT",
+        "gpl" : ["PreRasterization", "FragmentShader", "FragmentOutput"],
     },
     "VK_DYNAMIC_STATE_RAY_TRACING_PIPELINE_STACK_SIZE_KHR" : {
         "command" : ["vkCmdSetRayTracingPipelineStackSizeKHR"],
+        "stage" : "VK_SHADER_STAGE_ALL",
     },
     "VK_DYNAMIC_STATE_DEPTH_CLAMP_RANGE_EXT" : {
         "command" : ["vkCmdSetDepthClampRangeEXT"],
-        "dependency" : ["depthClampEnable"]
+        "dependency" : ["depthClampEnable"],
+        "stage" : "VK_SHADER_STAGE_ALL",
+        "gpl" : ["PreRasterization"],
+        "pipeline" : "VkPipelineViewportDepthClampControlCreateInfoEXT"
     },
 }
 
@@ -291,8 +515,8 @@ class DynamicStateOutputGenerator(BaseGenerator):
 
             /***************************************************************************
             *
-            * Copyright (c) 2023-2024 Valve Corporation
-            * Copyright (c) 2023-2024 LunarG, Inc.
+            * Copyright (c) 2023-2025 Valve Corporation
+            * Copyright (c) 2023-2025 LunarG, Inc.
             *
             * Licensed under the Apache License, Version 2.0 (the "License");
             * you may not use this file except in compliance with the License.
@@ -322,6 +546,7 @@ class DynamicStateOutputGenerator(BaseGenerator):
         out = []
         out.append('''
             #pragma once
+            #include <vulkan/vulkan_core.h>
             #include <bitset>
 
             namespace vvl {
@@ -349,7 +574,33 @@ class DynamicStateOutputGenerator(BaseGenerator):
 
             std::string DescribeDynamicStateCommand(CBDynamicState dynamic_state);
             std::string DescribeDynamicStateDependency(CBDynamicState dynamic_state, const vvl::Pipeline* pipeline);
+
+            // We build these up at code gen time to quickly use at runtime
+            // TODO - tried to make these constexpr, but seems bitset over 64 bits doesn't like that it seems
+            //
             ''')
+
+        const_names = {
+            "VK_SHADER_STAGE_VERTEX_BIT" : "kVertexDynamicState",
+            "VK_SHADER_STAGE_FRAGMENT_BIT" : "kFragmentDynamicState",
+            "VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT" : "kTessControlDynamicState",
+            "VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT" : "kTessEvalDynamicState",
+            "VK_SHADER_STAGE_GEOMETRY_BIT" : "kGeometryDynamicState",
+        }
+        stage_map = dict()
+        for key, value in dynamic_state_map.items():
+            stage_map.setdefault(value["stage"], []).append(key)
+
+        # sort so each OS generates same order here
+        for stage, states in sorted(stage_map.items()):
+            if stage == 'VK_SHADER_STAGE_ALL':
+                continue
+            out.append(f'// All state is only tied to {stage}\n')
+            out.append(f'const CBDynamicFlags {const_names[stage]} =\n')
+            for index, state in enumerate(states):
+                seperator = ' |' if (index + 1) != len(states) else ';\n'
+                out.append(f'CBDynamicFlags(1) << CB_{state[3:]}{seperator}\n')
+
         self.write("".join(out))
 
     def generateSource(self):
@@ -424,7 +675,7 @@ class DynamicStateOutputGenerator(BaseGenerator):
 
         out.append('''
             std::string DescribeDynamicStateCommand(CBDynamicState dynamic_state) {
-                std::stringstream ss;
+                std::ostringstream ss;
                 vvl::Func func = vvl::Func::Empty;
                 switch (dynamic_state) {
         ''')
@@ -456,7 +707,7 @@ class DynamicStateOutputGenerator(BaseGenerator):
             static std::string_view stencil_test_enable_static{"VkPipelineDepthStencilStateCreateInfo::stencilTestEnable was VK_TRUE in the last bound graphics pipeline.\\n"};
 
             std::string DescribeDynamicStateDependency(CBDynamicState dynamic_state, const vvl::Pipeline* pipeline) {
-                std::stringstream ss;
+                std::ostringstream ss;
                 switch (dynamic_state) {
         ''')
         for key, value in dynamic_state_map.items():
