@@ -1,9 +1,10 @@
 #!/usr/bin/python3 -i
 #
-# Copyright (c) 2015-2025 Valve Corporation
-# Copyright (c) 2015-2025 LunarG, Inc.
-# Copyright (c) 2015-2025 Google Inc.
+# Copyright (c) 2015-2026 Valve Corporation
+# Copyright (c) 2015-2026 LunarG, Inc.
+# Copyright (c) 2015-2026 Google Inc.
 # Copyright (c) 2023-2024 RasterGrid Kft.
+# Copyright (C) 2025 Arm Limited.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -65,7 +66,6 @@ class APISpecific:
 
 class LayerChassisOutputGenerator(BaseGenerator):
     ignore_functions = (
-        'vkEnumerateInstanceVersion',
     )
 
     manual_functions = (
@@ -76,6 +76,7 @@ class LayerChassisOutputGenerator(BaseGenerator):
         'vkDestroyDevice',
         'vkCreateInstance',
         'vkDestroyInstance',
+        'vkEnumerateInstanceVersion',
         'vkEnumerateInstanceLayerProperties',
         'vkEnumerateInstanceExtensionProperties',
         'vkEnumerateDeviceLayerProperties',
@@ -86,6 +87,7 @@ class LayerChassisOutputGenerator(BaseGenerator):
         'vkCreateComputePipelines',
         'vkCreateRayTracingPipelinesNV',
         'vkCreateRayTracingPipelinesKHR',
+        'vkCreateDataGraphPipelinesARM',
         'vkCreatePipelineLayout',
         'vkCreateShaderModule',
         'vkCreateShadersEXT',
@@ -93,6 +95,7 @@ class LayerChassisOutputGenerator(BaseGenerator):
         'vkAllocateDescriptorSets',
         'vkCreateBuffer',
         'vkQueuePresentKHR',
+        'vkCmdBindDescriptorBuffersEXT',
         # Need to inject HandleData logic
         'vkBeginCommandBuffer',
         # ValidationCache functions do not get dispatched
@@ -128,9 +131,9 @@ class LayerChassisOutputGenerator(BaseGenerator):
 
             /***************************************************************************
             *
-            * Copyright (c) 2015-2025 The Khronos Group Inc.
-            * Copyright (c) 2015-2025 Valve Corporation
-            * Copyright (c) 2015-2025 LunarG, Inc.
+            * Copyright (c) 2015-2026 The Khronos Group Inc.
+            * Copyright (c) 2015-2026 Valve Corporation
+            * Copyright (c) 2015-2026 LunarG, Inc.
             * Copyright (c) 2015-2024 Google Inc.
             * Copyright (c) 2023-2024 RasterGrid Kft.
             *
@@ -252,12 +255,19 @@ class LayerChassisOutputGenerator(BaseGenerator):
         out.append('''
             static const VkLayerProperties global_layer = {
                 OBJECT_LAYER_NAME,
-                VK_LAYER_API_VERSION,
+                VK_HEADER_VERSION_COMPLETE,
                 1,
                 "LunarG validation Layer",
             };
 
             // These functions reference generated data so they cannot be part of chassis_main.cpp
+            VKAPI_ATTR VkResult VKAPI_CALL EnumerateInstanceVersion(uint32_t* pApiVersion) {
+                if (pApiVersion) {
+                    *pApiVersion = VK_HEADER_VERSION_COMPLETE;
+                }
+                return VK_SUCCESS;
+            }
+
             VKAPI_ATTR VkResult VKAPI_CALL EnumerateInstanceLayerProperties(uint32_t* pCount, VkLayerProperties* pProperties) {
                 return util_GetLayerProperties(1, &global_layer, pCount, pProperties);
             }
@@ -345,6 +355,10 @@ class LayerChassisOutputGenerator(BaseGenerator):
             # Generate pre-call state recording source code
             out.append(f'RecordObject record_obj(vvl::Func::{command.name});\n')
             out.append(f'{{\nVVL_ZoneScopedN("PreCallRecord_{command.name}");')
+            if "QueueSubmit" in command.name:
+                out.append(f'''
+                    VVL_TracyVkNamedZoneStart(GetTracyVkCtx(), queue, "gpu_PreCallRecord{command.name}", pre_call_record_gpu_zone);
+                ''')
 
             if not command.instance:
                 out.append(f'''
@@ -364,6 +378,11 @@ class LayerChassisOutputGenerator(BaseGenerator):
 
             out.append(f'vo->PreCallRecord{command.name[2:]}({paramsList}, record_obj);\n')
             out.append('   }\n')
+            if "QueueSubmit" in command.name:
+                out.append('''
+                    VVL_TracyVkNamedZoneEnd(pre_call_record_gpu_zone, queue);
+                ''')
+
             out.append('}\n')
 
             # Insert pre-dispatch debug utils function call
@@ -384,8 +403,8 @@ class LayerChassisOutputGenerator(BaseGenerator):
             out.append(f'{{\nVVL_ZoneScopedN("Dispatch_{command.name}");\n')
 
             if "QueueSubmit" in command.name:
-                out.append('''
-                    VVL_TracyVkNamedZoneStart(GetTracyVkCtx(), queue, "gpu_QueueSubmit", submit_gpu_zone);
+                out.append(f'''
+                    VVL_TracyVkNamedZoneStart(GetTracyVkCtx(), queue, "gpu_{command.name}", submit_gpu_zone);
                 ''')
             assignResult = 'result = ' if (command.returnType != 'void') else ''
             method_name = command.name.replace('vk', f'{dispatch}->')
@@ -396,10 +415,6 @@ class LayerChassisOutputGenerator(BaseGenerator):
                 out.append('''
                     VVL_TracyVkNamedZoneEnd(submit_gpu_zone, queue);
                 ''')
-                out.append('''#if defined(VVL_TRACY_GPU)
-                    TracyVkCollector::TrySubmitCollectCb(queue);
-                #endif
-                ''')
             out.append('}\n')
 
             # Insert post-dispatch debug utils function call
@@ -420,6 +435,10 @@ class LayerChassisOutputGenerator(BaseGenerator):
 
             # Generate post-call object processing source code
             out.append(f'{{\nVVL_ZoneScopedN("PostCallRecord_{command.name}");\n')
+            if "QueueSubmit" in command.name:
+                out.append(f'''
+                    VVL_TracyVkNamedZoneStart(GetTracyVkCtx(), queue, "gpu_PostCallRecord{command.name}", post_call_record_gpu_zone);
+                ''')
 
             # Because each intercept is a copy of vvl::base::Device, we need to update it for each,
             # even if they don't intercept this command.
@@ -464,7 +483,16 @@ class LayerChassisOutputGenerator(BaseGenerator):
 
             out.append(f'vo->PostCallRecord{command.name[2:]}({paramsList}, record_obj);\n')
             out.append('    }\n')
+            if "QueueSubmit" in command.name:
+                out.append('''
+                    VVL_TracyVkNamedZoneEnd(post_call_record_gpu_zone, queue);
+                ''')
             out.append('}\n')
+            if "QueueSubmit" in command.name:
+                out.append('''#if defined(VVL_TRACY_GPU)
+                    TracyVkCollector::TrySubmitCollectCb(queue);
+                #endif
+                ''')
 
             # Return result variable, if any.
             if command.returnType != 'void':
