@@ -128,7 +128,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                             notSupported = False
                             break
                     if notSupported:
-                        continue # If just 'Kernel' capabilites then it's ment for OpenCL and skip instruction
+                        continue # If just 'Kernel' capabilites then it's meant for OpenCL and skip instruction
 
                 self.opnames.append(opname)
 
@@ -146,13 +146,13 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                     self.atomicsOps.append(opname)
                 if instruction['class'] == 'Non-Uniform':
                     self.groupOps.append(opname)
-                if re.search("OpImage.*Gather", opname) is not None:
+                if re.search(r'OpImage.*Gather', opname) is not None:
                     self.imageGatherOps.append(opname)
-                if re.search("OpImageFetch.*", opname) is not None:
+                if re.search(r'OpImageFetch.*', opname) is not None:
                     self.imageFetchOps.append(opname)
-                if re.search("OpImageSample.*", opname) is not None:
+                if re.search(r'OpImageSample.*', opname) is not None:
                     self.imageSampleOps.append(opname)
-                if re.search("OpType.*", opname) is not None:
+                if re.search(r'OpType.*', opname) is not None:
                     # Currently this is for GPU-AV which doesn't supporrt provisional extensions
                     if opname not in self.provisionalList:
                         self.typeOps.append(opname)
@@ -189,11 +189,11 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                         # some instructions have both types of IdScope
                         # OpReadClockKHR has the wrong 'name' as 'Scope'
                         if kind == 'IdScope':
-                            if operand['name'] == '\'Execution\'' or operand['name'] == '\'Scope\'':
+                            if operand['name'] == 'Execution' or operand['name'] == 'Scope':
                                 self.executionScopePosition[index + 1].append(opname)
-                            elif operand['name'] == '\'Memory\'':
+                            elif operand['name'] == 'Memory':
                                 self.memoryScopePosition[index + 1].append(opname)
-                            elif operand['name'] == '\'Visibility\'':
+                            elif operand['name'] == 'Visibility':
                                 continue # ignore
                             else:
                                 print(f'Error: unknown operand {opname} with IdScope {operand["name"]} not handled correctly\n')
@@ -201,12 +201,12 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                         if kind == 'ImageOperands':
                             self.imageOperandsPosition[index + 1].append(opname)
                         if kind == 'IdRef':
-                            if operand['name'] == '\'Image\'':
+                            if operand['name'] == 'Image':
                                 self.opcodes[opcode]['imageRefPosition'] = index + 1
-                            elif operand['name'] == '\'Sampled Image\'':
+                            elif operand['name'] == 'Sampled Image':
                                 self.opcodes[opcode]['sampledImageRefPosition'] = index + 1
 
-                if re.search("OpImage*", opname) is not None:
+                if re.search(r'OpImage*', opname) is not None:
                     info = self.opcodes[opcode]
                     imageRef = info['imageRefPosition']
                     sampledImageRef = info['sampledImageRefPosition']
@@ -286,7 +286,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
             const char* string_SpvExecutionModel(uint32_t execution_model);
             const char* string_SpvExecutionMode(uint32_t execution_mode);
             const char* string_SpvDecoration(uint32_t decoration);
-            const char* string_SpvBuiltIn(uint32_t built_in);
+            const char* string_SpvBuiltIn(spv::BuiltIn built_in);
             const char* string_SpvDim(uint32_t dim);
             std::string string_SpvCooperativeMatrixOperands(uint32_t mask);
             ''')
@@ -596,7 +596,7 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
                 }}
             }}
 
-            const char* string_SpvBuiltIn(uint32_t built_in) {{
+            const char* string_SpvBuiltIn(spv::BuiltIn built_in) {{
                 switch(built_in) {{
             {"".join([f"""        case spv::BuiltIn{x}:
                         return "{x}";
@@ -646,7 +646,6 @@ class SpirvGrammarHelperOutputGenerator(BaseGenerator):
             }}
             ''')
 
-
         out.append('''
             const OperandInfo& GetOperandInfo(uint32_t opcode) {
                 static const vvl::unordered_map<uint32_t, OperandInfo> kOperandTable {
