@@ -34,7 +34,7 @@ TEST_F(NegativeDebugExtensions, DebugMarkerName) {
     vkt::Buffer buffer(*m_device, buffer_create_info, vkt::no_mem);
 
     VkMemoryRequirements memRequirements;
-    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &memRequirements);
+    vk::GetBufferMemoryRequirements(device(), buffer, &memRequirements);
 
     VkMemoryAllocateInfo memory_allocate_info = vku::InitStructHelper();
     memory_allocate_info.allocationSize = memRequirements.size;
@@ -48,11 +48,11 @@ TEST_F(NegativeDebugExtensions, DebugMarkerName) {
     name_info.pObjectName = memory_name.c_str();
     vk::DebugMarkerSetObjectNameEXT(device(), &name_info);
 
-    vk::BindBufferMemory(device(), buffer.handle(), memory_1.handle(), 0);
+    vk::BindBufferMemory(device(), buffer, memory_1, 0);
 
     // Test core_validation layer
     m_errorMonitor->SetDesiredError(memory_name.c_str());
-    vk::BindBufferMemory(device(), buffer.handle(), memory_2.handle(), 0);
+    vk::BindBufferMemory(device(), buffer, memory_2, 0);
     m_errorMonitor->VerifyFound();
 
     VkCommandBuffer commandBuffer;
@@ -64,7 +64,7 @@ TEST_F(NegativeDebugExtensions, DebugMarkerName) {
     vkt::CommandPool command_pool_2(*m_device, pool_create_info);
 
     VkCommandBufferAllocateInfo command_buffer_allocate_info = vku::InitStructHelper();
-    command_buffer_allocate_info.commandPool = command_pool_1.handle();
+    command_buffer_allocate_info.commandPool = command_pool_1;
     command_buffer_allocate_info.commandBufferCount = 1;
     command_buffer_allocate_info.level = VK_COMMAND_BUFFER_LEVEL_PRIMARY;
     vk::AllocateCommandBuffers(device(), &command_buffer_allocate_info, &commandBuffer);
@@ -88,7 +88,7 @@ TEST_F(NegativeDebugExtensions, DebugMarkerName) {
 
     // Test object_tracker layer
     m_errorMonitor->SetDesiredError(commandBuffer_name.c_str());
-    vk::FreeCommandBuffers(device(), command_pool_2.handle(), 1, &commandBuffer);
+    vk::FreeCommandBuffers(device(), command_pool_2, 1, &commandBuffer);
     m_errorMonitor->VerifyFound();
 }
 
@@ -139,9 +139,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     }
 
     DebugUtilsLabelCheckData callback_data;
-    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *pCallbackData, DebugUtilsLabelCheckData *data) {
-        data->count++;
-    };
+    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *, DebugUtilsLabelCheckData *data) { data->count++; };
     callback_data.count = 0;
     callback_data.callback = empty_callback;
 
@@ -162,7 +160,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     vkt::Buffer buffer(*m_device, buffer_create_info, vkt::no_mem);
 
     VkMemoryRequirements memRequirements;
-    vk::GetBufferMemoryRequirements(device(), buffer.handle(), &memRequirements);
+    vk::GetBufferMemoryRequirements(device(), buffer, &memRequirements);
 
     VkMemoryAllocateInfo memory_allocate_info = vku::InitStructHelper();
     memory_allocate_info.allocationSize = memRequirements.size;
@@ -197,11 +195,11 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     name_info.objectType = VK_OBJECT_TYPE_DEVICE_MEMORY;
     vk::SetDebugUtilsObjectNameEXT(device(), &name_info);
 
-    vk::BindBufferMemory(device(), buffer.handle(), memory_1.handle(), 0);
+    vk::BindBufferMemory(device(), buffer, memory_1, 0);
 
     // Test core_validation layer
     m_errorMonitor->SetDesiredError(memory_name.c_str());
-    vk::BindBufferMemory(device(), buffer.handle(), memory_2.handle(), 0);
+    vk::BindBufferMemory(device(), buffer, memory_2, 0);
     m_errorMonitor->VerifyFound();
 
     VkCommandBuffer commandBuffer;
@@ -213,7 +211,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
     vkt::CommandPool command_pool_2(*m_device, pool_create_info);
 
     VkCommandBufferAllocateInfo command_buffer_allocate_info = vku::InitStructHelper();
-    command_buffer_allocate_info.commandPool = command_pool_1.handle();
+    command_buffer_allocate_info.commandPool = command_pool_1;
     command_buffer_allocate_info.commandBufferCount = 1;
     command_buffer_allocate_info.level = VK_COMMAND_BUFFER_LEVEL_PRIMARY;
     vk::AllocateCommandBuffers(device(), &command_buffer_allocate_info, &commandBuffer);
@@ -260,7 +258,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsName) {
 
     // Test object_tracker layer
     m_errorMonitor->SetDesiredError(commandBuffer_name.c_str());
-    vk::FreeCommandBuffers(device(), command_pool_2.handle(), 1, &commandBuffer);
+    vk::FreeCommandBuffers(device(), command_pool_2, 1, &commandBuffer);
     m_errorMonitor->VerifyFound();
 
     vk::DestroyDebugUtilsMessengerEXT(instance(), my_messenger, nullptr);
@@ -308,9 +306,7 @@ TEST_F(NegativeDebugExtensions, DebugUtilsParameterFlags) {
     RETURN_IF_SKIP(Init());
 
     DebugUtilsLabelCheckData callback_data;
-    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *pCallbackData, DebugUtilsLabelCheckData *data) {
-        data->count++;
-    };
+    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *, DebugUtilsLabelCheckData *data) { data->count++; };
     callback_data.count = 0;
     callback_data.callback = empty_callback;
 
@@ -380,12 +376,10 @@ TEST_F(NegativeDebugExtensions, SetDebugUtilsObjectSecondDevice) {
     }
 
     auto features = m_device->Physical().Features();
-    vkt::Device second_device(gpu_, m_device_extension_names, &features, nullptr);
+    vkt::Device second_device(gpu_, m_device_extension_names, &features);
 
     DebugUtilsLabelCheckData callback_data;
-    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *pCallbackData, DebugUtilsLabelCheckData *data) {
-        data->count++;
-    };
+    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *, DebugUtilsLabelCheckData *data) { data->count++; };
     callback_data.count = 0;
     callback_data.callback = empty_callback;
 
@@ -421,9 +415,7 @@ TEST_F(NegativeDebugExtensions, SetDebugUtilsObjectDestroyedHandle) {
     }
 
     DebugUtilsLabelCheckData callback_data;
-    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *pCallbackData, DebugUtilsLabelCheckData *data) {
-        data->count++;
-    };
+    auto empty_callback = [](const VkDebugUtilsMessengerCallbackDataEXT *, DebugUtilsLabelCheckData *data) { data->count++; };
     callback_data.count = 0;
     callback_data.callback = empty_callback;
 
@@ -438,7 +430,7 @@ TEST_F(NegativeDebugExtensions, SetDebugUtilsObjectDestroyedHandle) {
 
     vkt::Sampler sampler(*m_device, SafeSaneSamplerCreateInfo());
     uint64_t bad_handle = (uint64_t)sampler.handle();
-    sampler.destroy();
+    sampler.Destroy();
     const char *object_name = "sampler_object";
 
     VkDebugUtilsObjectNameInfoEXT name_info = vku::InitStructHelper();
@@ -538,11 +530,11 @@ TEST_F(NegativeDebugExtensions, SwapchainImagesDebugMarker) {
     RETURN_IF_SKIP(Init());
     RETURN_IF_SKIP(InitSurface());
 
-    SurfaceInformation info = GetSwapchainInfo(m_surface.Handle());
+    SurfaceInformation info = GetSwapchainInfo(m_surface);
     InitSwapchainInfo();
 
     VkSwapchainCreateInfoKHR swapchain_create_info = vku::InitStructHelper();
-    swapchain_create_info.surface = m_surface.Handle();
+    swapchain_create_info.surface = m_surface;
     swapchain_create_info.minImageCount = info.surface_capabilities.minImageCount;
     swapchain_create_info.imageFormat = info.surface_formats[0].format;
     swapchain_create_info.imageColorSpace = info.surface_formats[0].colorSpace;
@@ -663,3 +655,42 @@ TEST_F(NegativeDebugExtensions, SetDeviceHandle) {
     vk::DestroyBuffer(device(), buffer, nullptr);
     vk::DestroyDevice(second_device, nullptr);
 }
+
+TEST_F(NegativeDebugExtensions, DebugMarkerRecording) {
+    AddRequiredExtensions(VK_EXT_DEBUG_REPORT_EXTENSION_NAME);
+    AddRequiredExtensions(VK_EXT_DEBUG_MARKER_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    VkDebugMarkerMarkerInfoEXT marker_info = vku::InitStructHelper();
+    marker_info.pMarkerName = "test";
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDebugMarkerBeginEXT-commandBuffer-recording");
+    vk::CmdDebugMarkerBeginEXT(m_command_buffer, &marker_info);
+    m_errorMonitor->VerifyFound();
+
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDebugMarkerEndEXT-commandBuffer-recording");
+    vk::CmdDebugMarkerEndEXT(m_command_buffer);
+    m_errorMonitor->VerifyFound();
+
+    m_errorMonitor->SetDesiredError("VUID-vkCmdDebugMarkerInsertEXT-commandBuffer-recording");
+    vk::CmdDebugMarkerInsertEXT(m_command_buffer, &marker_info);
+    m_errorMonitor->VerifyFound();
+}
+
+TEST_F(NegativeDebugExtensions, DebugUtilsRecording) {
+    AddRequiredExtensions(VK_EXT_DEBUG_UTILS_EXTENSION_NAME);
+    RETURN_IF_SKIP(Init());
+
+    VkDebugUtilsLabelEXT label = vku::InitStructHelper();
+    label.pLabelName = "test";
+    m_errorMonitor->SetDesiredError("VUID-vkCmdBeginDebugUtilsLabelEXT-commandBuffer-recording");
+    vk::CmdBeginDebugUtilsLabelEXT(m_command_buffer, &label);
+    m_errorMonitor->VerifyFound();
+
+    m_errorMonitor->SetDesiredError("VUID-vkCmdEndDebugUtilsLabelEXT-commandBuffer-recording");
+    vk::CmdEndDebugUtilsLabelEXT(m_command_buffer);
+    m_errorMonitor->VerifyFound();
+
+    m_errorMonitor->SetDesiredError("VUID-vkCmdInsertDebugUtilsLabelEXT-commandBuffer-recording");
+    vk::CmdInsertDebugUtilsLabelEXT(m_command_buffer, &label);
+    m_errorMonitor->VerifyFound();
+}
